#!/usr/bin/env python

import os
import sys
import unittest

import numpy

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

from hienoi._kdtree import KDTree


class KDTreeTest(unittest.TestCase):

    def test_search_1(self):
        dtype = numpy.dtype((numpy.float32, 2))
        a = numpy.array([
            ( 1.1,  1.5),
            ( 1.2, -1.6),
            (-1.3, -1.7),
            (-1.4,  1.8),
            ( 0.5,  0.5),
        ], dtype=dtype)

        suite = [
            {
                'searches': [(((2.0, 4.0),), {'count': n, 'radius': None, 'sort': True}) for n in range(len(a) + 1)],
                'expected_indices': [0, 4, 3, 1, 2],
                'expected_squared_distances': [7.06, 14.5, 16.4, 32.0, 43.38],
            },
            {
                'searches': [(((0.6, -0.7),), {'count': n, 'radius': None, 'sort': True}) for n in range(len(a) + 1)],
                'expected_indices': [1, 4, 2, 0, 3],
                'expected_squared_distances': [1.17, 1.45, 4.61, 5.09, 10.25],
            },
            {
                'searches': [(((-0.2, 0.4),), {'count': len(a), 'radius': 1.0, 'sort': True})],
                'expected_indices': [4],
                'expected_squared_distances': [0.5],
            },
            {
                'searches': [(((-0.2, 0.4),), {'count': len(a), 'radius': 2.0, 'sort': True})],
                'expected_indices': [4, 0, 3],
                'expected_squared_distances': [0.5, 2.9, 3.4],
            },
            {
                'searches': [(((-0.2, 0.4),), {'count': len(a), 'radius': 3.0, 'sort': True})],
                'expected_indices': [4, 0, 3, 2, 1],
                'expected_squared_distances': [0.5, 2.9, 3.4, 5.62, 5.96],
            },
        ]

        trees = [KDTree(a, bucket_size=size) for size in range(1, len(a) + 1)]
        for tree in trees:
            self._test_search_suite(tree, suite)

    def test_search_2(self):
        dtype = numpy.dtype((numpy.float32, 2))
        a = numpy.array([
            ( 1.1,  1.5),
            ( 1.2,  0.5),
            (-1.3,  0.5),
            (-1.4,  1.8),
            ( 0.5,  0.5),
        ], dtype=dtype)

        suite = [
            {
                'searches': [(((-0.2, -1.5),), {'count': n, 'radius': None, 'sort': True}) for n in range(len(a) + 1)],
                'expected_indices': [4, 2, 1, 0, 3],
                'expected_squared_distances': [4.49, 5.21, 5.96, 10.69, 12.33],
            },
            {
                'searches': [(((1.3, -0.1),), {'count': n, 'radius': None, 'sort': True}) for n in range(len(a) + 1)],
                'expected_indices': [1, 4, 0, 2, 3],
                'expected_squared_distances': [0.37, 1.0, 2.6, 7.12, 10.9],
            },
            {
                'searches': [(((2.1, 0.7),), {'count': len(a), 'radius': 1.0, 'sort': True})],
                'expected_indices': [1],
                'expected_squared_distances': [0.85],
            },
            {
                'searches': [(((2.1, 0.7),), {'count': len(a), 'radius': 2.0, 'sort': True})],
                'expected_indices': [1, 0, 4],
                'expected_squared_distances': [0.85, 1.64, 2.6],
            },
            {
                'searches': [(((2.1, 0.7),), {'count': len(a), 'radius': 3.0, 'sort': True})],
                'expected_indices': [1, 0, 4],
                'expected_squared_distances': [0.85, 1.64, 2.6],
            },
            {
                'searches': [(((2.1, 0.7),), {'count': len(a), 'radius': 4.0, 'sort': True})],
                'expected_indices': [1, 0, 4, 2, 3],
                'expected_squared_distances': [0.85, 1.64, 2.6, 11.6, 13.46],
            },
        ]

        trees = [KDTree(a, bucket_size=size) for size in range(1, len(a) + 1)]
        for tree in trees:
            self._test_search_suite(tree, suite)

    def _test_search_suite(self, tree, suite):
        for case in suite:
            for search in case['searches']:
                args, kwargs = search[0], search[1]
                n = min(len(case['expected_indices']), kwargs['count'])

                neighbours = tree.search(*args, **kwargs)
                self.assertEqual(len(neighbours), n)
                self.assertEqual(neighbours['index'].tolist(), case['expected_indices'][:n])
                self.assertEqual([round(d, 3) for d in neighbours['squared_distance']], case['expected_squared_distances'][:n])


if __name__ == '__main__':
    from tests.run import run
    run('__main__')



"""Kd-tree for nearest neighbours queries.

This implementation currently runs faster than SciPy's KDTree but is often
slower than a brute-force approach in Hienoi's use cases, killing the primary
purpose for such a data structure. Still, it is left here as an educational
resource.
"""

import collections
import heapq
import itertools
import sys

import numpy

from hienoi._dynamicarray import DynamicArray


if sys.version_info[0] == 2:
    _range = xrange
    _zip = itertools.izip
else:
    _range = range
    _zip = zip

_heappushpop = heapq.heappushpop
_heappush = heapq.heappush
_npmax = numpy.maximum


_INITIAL_NEIGHBOURS_CAPACITY = 32


class KDTree(object):
    """K-d tree.

    Nodes are stored contiguously in memory.
    """

    def __init__(self, data, bucket_size=128):
        if bucket_size < 1:
            raise ValueError("A minimum bucket size of 1 is expected.")

        self._data = data
        self._n, self._k = self._data.shape
        self._nodes = None
        self._buckets = []
        self._bucket_size = bucket_size

        self._node_dtype = numpy.dtype([
            ('size', numpy.intp),
            ('bucket', numpy.intp),
            ('lower_bounds', (numpy.float_, self._k)),
            ('upper_bounds', (numpy.float_, self._k)),
        ])
        self._neighbour_dtype = numpy.dtype([
            ('squared_distance', numpy.float_),
            ('index', numpy.intp),
        ])

        self._build()

    def search(self, point, count, radius, sort):
        """Retrieve the neighbours to a point."""
        if count is None:
            count = self._n
        elif count < 1:
            return numpy.empty(0, dtype=self._neighbour_dtype)

        if radius is None:
            radius = numpy.inf
        elif radius < 0.0:
            return numpy.empty(0, dtype=self._neighbour_dtype)

        point = numpy.asarray(point, dtype=numpy.float_)
        if count >= self._n:
            return self._search_all_within_radius(point, radius, sort)
        else:
            return self._search_k_nearests(point, count, radius, sort)

    def _build(self):
        """Build the k-d tree."""
        data = self._data
        buckets = self._buckets
        bucket_size = self._bucket_size

        # First pass: build the tree using a DFS ordering.
        nodes = []
        parents = []
        i = 0
        stack = collections.deque(((numpy.arange(self._n), None),))
        while stack:
            indices, parent = stack.popleft()
            points = data[indices]
            lower_bounds = numpy.amin(points, axis=0)
            upper_bounds = numpy.amax(points, axis=0)
            if len(points) <= bucket_size:
                bucket = len(buckets)
                buckets.append(indices)
            else:
                bucket = -1

                # Split the longest side of the bounds.
                side_lengths = upper_bounds - lower_bounds
                split_axis = numpy.argmax(side_lengths)
                split_location = (lower_bounds[split_axis]
                                  + upper_bounds[split_axis]) / 2.0
                axis_data = points[:, split_axis]
                left_indices = indices[
                    numpy.nonzero(axis_data <= split_location)[0]]
                right_indices = indices[
                    numpy.nonzero(axis_data > split_location)[0]]
                stack.appendleft((right_indices, i))
                stack.appendleft((left_indices, i))

            size = 1
            nodes.append((size, bucket, lower_bounds, upper_bounds))
            parents.append(parent)
            i += 1

        # Second pass: set the 'size' attribute for each node in the tree.
        # Iterate over the nodes in reverse order to perform a bottom-up
        # traversal where the size of each node is added to the size of its
        # parent.
        self._nodes = numpy.array(nodes, dtype=self._node_dtype)
        node_sizes = self._nodes['size']
        for i in reversed(_range(1, len(self._nodes))):
            node_sizes[parents[i]] += node_sizes[i]

    def _search_k_nearests(self, point, count, radius, sort):
        """Search the nearest points within a radius."""
        data = self._data
        nodes = self._nodes
        buckets = self._buckets

        node_sizes = nodes['size']
        node_buckets = nodes['bucket']
        node_lower_bounds = nodes['lower_bounds']
        node_upper_bounds = nodes['upper_bounds']

        # A max heap would be appropriate to store the nearest points found,
        # alas Python only provides a min heap as part of the `heapq` module.
        # As a hacky workaround, nearest distances are negated.
        nearests = []
        dist_limit = radius ** 2

        pt_root_dist = _pt_to_node_near_dist(
            point, node_lower_bounds[0], node_upper_bounds[0])
        stack = collections.deque(((0, pt_root_dist),))
        while stack:
            i, pt_node_dist = stack.popleft()
            if pt_node_dist > dist_limit:
                # The node's bounds are too far, skip this branch.
                pass
            elif node_sizes[i] == 1:
                # This is a leaf node, see if there are any nearest points.
                indices = buckets[node_buckets[i]]
                points = data[indices]
                dists = numpy.sum(
                    (point[numpy.newaxis, :] - points) ** 2, axis=-1)
                for j, dist in _zip(indices, dists):
                    if dist < dist_limit:
                        if len(nearests) >= count:
                            dist_limit = -_heappushpop(nearests, (-dist, j))[0]
                        else:
                            _heappush(nearests, (-dist, j))
            else:
                # Inspect the child nodes.
                left_node_idx = i + 1
                right_node_idx = i + 1 + node_sizes[left_node_idx]
                pt_left_node_dist = _pt_to_node_near_dist(
                    point, node_lower_bounds[left_node_idx],
                    node_upper_bounds[left_node_idx])
                pt_right_node_dist = _pt_to_node_near_dist(
                    point, node_lower_bounds[right_node_idx],
                    node_upper_bounds[right_node_idx])
                if pt_left_node_dist <= pt_right_node_dist:
                    stack.appendleft((right_node_idx, pt_right_node_dist))
                    stack.appendleft((left_node_idx, pt_left_node_dist))
                else:
                    stack.appendleft((left_node_idx, pt_left_node_dist))
                    stack.appendleft((right_node_idx, pt_right_node_dist))

        out = numpy.array(nearests, dtype=self._neighbour_dtype)
        out['squared_distance'] *= -1.0
        if sort:
            # Here is the biggest performance killer. Runs in a single thread.
            out.sort(order='squared_distance')

        return out

    def _search_all_within_radius(self, point, radius, sort):
        """Search all the points within a radius."""
        data = self._data
        nodes = self._nodes
        buckets = self._buckets

        node_sizes = nodes['size']
        node_buckets = nodes['bucket']
        node_lower_bounds = nodes['lower_bounds']
        node_upper_bounds = nodes['upper_bounds']

        radius **= 2
        neighbours = DynamicArray(_INITIAL_NEIGHBOURS_CAPACITY,
                                  self._neighbour_dtype)

        pt_root_dist = _pt_to_node_near_dist(
            point, node_lower_bounds[0], node_upper_bounds[0])
        stack = collections.deque(((0, pt_root_dist),))
        while stack:
            i, pt_node_near_dist = stack.popleft()
            if pt_node_near_dist > radius:
                # The node's bounds are too far, skip this branch.
                pass
            elif (_pt_to_node_far_dist(
                    point, node_lower_bounds[i], node_upper_bounds[i])
                  <= radius):
                # The node's bounds are within the radius, recursively retrieve
                # all the points.
                children = nodes[i:i + node_sizes[i]]
                leaves = numpy.extract(children['size'] == 1, children)
                count = sum(len(buckets[bucket])
                            for bucket in leaves['bucket'])
                j = len(neighbours)
                neighbours.resize(j + count)
                neighbours.data['squared_distance'][j:] = numpy.nan
                for bucket in leaves['bucket']:
                    indices = buckets[bucket]
                    count = len(indices)
                    neighbours.data['index'][j:j + count] = indices
                    j += count
            elif node_sizes[i] == 1:
                # This is a leaf node, see if there are any points within the
                # radius.
                indices = buckets[node_buckets[i]]
                points = data[indices]
                dists = numpy.sum(
                    (point[numpy.newaxis, :] - points) ** 2, axis=-1)
                is_within = dists <= radius
                j = len(neighbours)
                neighbours.resize(j + numpy.sum(is_within))
                neighbours.data['squared_distance'][j:] = dists[is_within]
                neighbours.data['index'][j:] = indices[is_within]
            else:
                # Inspect the child nodes.
                left_node_idx = i + 1
                right_node_idx = i + 1 + node_sizes[left_node_idx]
                pt_left_node_dist = _pt_to_node_near_dist(
                    point, node_lower_bounds[left_node_idx],
                    node_upper_bounds[left_node_idx])
                pt_right_node_dist = _pt_to_node_near_dist(
                    point, node_lower_bounds[right_node_idx],
                    node_upper_bounds[right_node_idx])
                if pt_left_node_dist <= pt_right_node_dist:
                    stack.appendleft((right_node_idx, pt_right_node_dist))
                    stack.appendleft((left_node_idx, pt_left_node_dist))
                else:
                    stack.appendleft((left_node_idx, pt_left_node_dist))
                    stack.appendleft((right_node_idx, pt_right_node_dist))

        # Compute all the distances.
        is_nan = numpy.isnan(neighbours.data['squared_distance'])
        indices = neighbours.data['index'][numpy.nonzero(is_nan)]
        points = data[indices]
        dists = numpy.sum((point[numpy.newaxis, :] - points) ** 2, axis=-1)
        neighbours.data['squared_distance'][is_nan] = dists

        if sort:
            # Here is the biggest performance killer. Runs in a single thread.
            neighbours.data.sort(order='squared_distance')

        return neighbours.data


def _pt_to_node_near_dist(point, lower_bounds, upper_bounds):
    return numpy.sum(
        _npmax(0.0, _npmax(point - upper_bounds, lower_bounds - point))
        ** 2)


def _pt_to_node_far_dist(point, lower_bounds, upper_bounds):
    return numpy.sum(
        _npmax(0.0, _npmax(upper_bounds - point, point - lower_bounds))
        ** 2)



#!/usr/bin/env python

"""Aggregation of particles around the origin.

Features:

- simulation callbacks: particles are initialized within a radius from the
  origin and are, at each simulation step, updated to move towards the origin
  while keeping a separation distance from each other.
- GUI event callback: on each left mouse button click, a new particle is added
  to the simulation under the mouses's location.
- neighbouring: each particle retrieve the neighbours within a radius to keep
  itself separated from the others.
"""

import math
import os
import random
import sys

import sdl2

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

import hienoi
import hienoi.application
from hienoi import Vector2f, Vector4f
from hienoi.application import Callback
from hienoi.gui import NavigationAction


if sys.version_info[0] == 2:
    _range = xrange
else:
    _range = range


# Number of particles.
_COUNT = 8
# Radius of the disc used to distribute the particles.
_RADIUS = 30.0
# Size of a particle.
_SIZE = 1.0
# Strength of the attraction towards the origin.
_ATTRACTION_FORCE = 1.0
# Threshold distance below which a particle starts separating from another.
_SEPARATION_DISTANCE = 5.0
# Strength of the separation force.
_SEPARATION_FORCE = 50.0


def _rand(seed=None):
    """Generate a pseudo-random number ranging from 0.0 to 1.0."""
    random.seed(seed)
    return random.random()


def _rand_0(seed=None):
    """Generate a pseudo-random number around 0, ranging from -1.0 to 1.0."""
    random.seed(seed)
    return random.random() * 2.0 - 1.0


def _remap_clamped(value, from_1, to_1, from_2, to_2):
    """Remap a value from a range to another."""
    result = ((value - from_1) / (to_1 - from_1)) * (to_2 - from_2) + from_2
    return sorted((from_2, result, to_2))[1]


def _add_particle(sim, position):
    """Inter-process callback to add a new particle to the simulation.

    The first parameter of inter-process callbacks is always reserved for
    the object to operate on, here the particle simulation object. Remaining
    parameters are reserved to the user.
    """
    sim.add_particle(position=position, size=_SIZE)


def on_gui_event(gui, data, event):
    """Callback to react to GUI events.

    Parameters
    ----------
    gui : hienoi.gui.GUI
        GUI.
    data : hienoi.application.OnEventData
        Data.
    event : sdl2.SDL_Event
        GUI event.
    """
    if event.type == sdl2.SDL_MOUSEBUTTONDOWN:
        button_event = event.button
        if (button_event.button == sdl2.SDL_BUTTON_LEFT
                and gui.navigation_action == NavigationAction.NONE):
            # Create an 'add_particle' callback to be run by the particle
            # simulation object.
            mouse_position = gui.get_mouse_position()
            position = gui.screen_to_world(mouse_position)
            callback = Callback(_add_particle, args=(position,))
            data.callbacks.particle_simulation.append(callback)


def initialize_particle_simulation(sim):
    """Callback to initialize the particle simulation state.

    Parameters
    ----------
    sim : hienoi.dynamics.ParticleSimulation
        Particle simulation.
    """
    # Add a few particles at random positions within a given radius and with
    # random velocities having a magnitude ranging from -1.0 to 1.0.
    for i in _range(_COUNT):
        r = _rand(seed=i + 0.827) * _RADIUS
        t = _rand(seed=i + 0.301) * 2.0 * math.pi
        sim.add_particle(position=Vector2f(r * math.cos(t), r * math.sin(t)),
                         size=_SIZE)


def update_particle_simulation(sim):
    """Callback to update the particle simulation state.

    Parameters
    ----------
    sim : hienoi.dynamics.ParticleSimulation
        Particle simulation.
    """
    for particle in sim.particles:
        # Compute the attractive force.
        force = -particle.position - particle.velocity
        force.iscale(_ATTRACTION_FORCE)
        particle.force += force

        # Compute the separative force.
        highest_proximity = 0.0
        radius = _SIZE * 2.0 + _SEPARATION_DISTANCE
        squared_radius = radius ** 2
        neighbours = sim.get_neighbour_particles(
            particle.position, count=None, radius=radius)
        for neighbour in neighbours:
            if neighbour.particle.id == particle.id:
                continue

            if neighbour.squared_distance > squared_radius:
                continue

            proximity = _remap_clamped(neighbour.distance,
                                       particle.size, radius,
                                       1.0, 0.0)
            highest_proximity = max(proximity, highest_proximity)

            force = particle.position - neighbour.particle.position
            force.iscale(proximity * _SEPARATION_FORCE)
            particle.force += force

        # Color the particle according to how near it is to any other particle.
        particle.color = Vector4f(1.0,
                                  1.0 - highest_proximity,
                                  1.0 - highest_proximity,
                                  1.0)


def run():
    """Run the application."""
    return hienoi.application.run(
        gui={
            'window_title': 'equilibrium',
            'show_grid': False,
            'on_event_callback': on_gui_event,
        },
        particle_simulation={
            'initialize_callback': initialize_particle_simulation,
            'postsolve_callback': update_particle_simulation,
        })


if __name__ == '__main__':
    sys.exit(run())



#!/usr/bin/env python

import os
import sys
import unittest

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

import hienoi.dynamics
from hienoi import Vector2f


class ParticleSimulationTest(unittest.TestCase):

    def test_constructor_1(self):
        sim = hienoi.dynamics.ParticleSimulation()
        self.assertEqual(sim.time, 0.0)
        self.assertEqual(sim.last_particle_id, -1)
        self.assertEqual(len(sim.particles), 0)

    def test_constructor_2(self):
        def initialize(sim):
            p0 = sim.add_particle()
            p1 = sim.add_particle()
            p2 = sim.add_particle()
            p1.alive = False

            self.assertEqual(sim.time, 0.0)
            self.assertEqual(sim.last_particle_id, 2)
            self.assertEqual(len(sim.particles), 0)
            self.assertEqual(len(list(iter(sim.particles))), 0)
            self.assertTrue(p0.alive)
            self.assertFalse(p1.alive)
            self.assertTrue(p2.alive)

        sim = hienoi.dynamics.ParticleSimulation(initialize_callback=initialize)
        self.assertEqual(sim.time, 0.0)
        self.assertEqual(sim.last_particle_id, 2)
        self.assertEqual(len(sim.particles), 2)
        self.assertEqual(len(list(iter(sim.particles))), 2)
        for particle in sim.particles:
            self.assertTrue(particle.alive)

    def test_add_particle(self):
        sim = hienoi.dynamics.ParticleSimulation(particle_bucket_buffer_capacity=2)

        p0 = sim.add_particle(size=2.0)
        self.assertEqual(len(sim.particles), 0)
        self.assertEqual(len(list(iter(sim.particles))), 0)
        self.assertEqual(sim.last_particle_id, 0)
        self.assertEqual(p0.id, 0)
        self.assertEqual(p0.size, 2.0)
        self.assertTrue(p0.alive)

        p1 = sim.add_particle(id=99)
        self.assertEqual(len(sim.particles), 0)
        self.assertEqual(len(list(iter(sim.particles))), 0)
        self.assertEqual(sim.last_particle_id, 1)
        self.assertEqual(p1.id, 1)
        self.assertTrue(p1.alive)

        p2 = sim.add_particle()
        self.assertEqual(len(sim.particles), 0)
        self.assertEqual(len(list(iter(sim.particles))), 0)
        self.assertEqual(sim.last_particle_id, 2)
        self.assertEqual(p2.id, 2)
        self.assertTrue(p2.alive)

        sim.consolidate()
        self.assertEqual(len(sim.particles), 3)
        self.assertEqual(len(list(iter(sim.particles))), 3)
        self.assertEqual(sim.last_particle_id, 2)

    def test_add_particles(self):
        sim = hienoi.dynamics.ParticleSimulation(particle_bucket_buffer_capacity=3)

        bunch = sim.add_particles(2)
        self.assertEqual(len(bunch), 2)
        self.assertEqual([particle.id for particle in bunch], [0, 1])
        self.assertEqual(len(sim.particles), 0)
        self.assertEqual(len(list(iter(sim.particles))), 0)
        self.assertEqual(sim.last_particle_id, 1)

        bunch = sim.add_particles(5)
        self.assertEqual(len(bunch), 5)
        self.assertEqual([particle.id for particle in bunch], [2, 3, 4, 5, 6])
        self.assertEqual(len(sim.particles), 0)
        self.assertEqual(len(list(iter(sim.particles))), 0)
        self.assertEqual(sim.last_particle_id, 6)

        sim.consolidate()
        self.assertEqual(len(sim.particles), 7)
        self.assertEqual(len(list(iter(sim.particles))), 7)
        self.assertEqual(sim.last_particle_id, 6)
        self.assertEqual([particle.id for particle in sim.particles], [0, 1, 2, 3, 4, 5, 6])

    def test_kill_particle(self):
        sim = hienoi.dynamics.ParticleSimulation()
        sim.add_particle()
        sim.add_particle()
        sim.consolidate()

        p0 = sim.get_particle(0)
        p1 = sim.get_particle(1)

        p2 = sim.add_particle()
        p3 = sim.add_particle()
        p4 = sim.add_particle()

        p0.alive = False
        p3.alive = False

        self.assertFalse(p0.alive)
        self.assertTrue(p1.alive)
        self.assertTrue(p2.alive)
        self.assertFalse(p3.alive)
        self.assertEqual(len(sim.particles), 2)

        sim.consolidate()
        self.assertEqual(len(sim.particles), 3)
        self.assertEqual([particle.id for particle in sim.particles], [1, 2, 4])

    def test_get_particle(self):
        sim = hienoi.dynamics.ParticleSimulation()

        sim.add_particle()
        sim.add_particle()
        self.assertRaises(ValueError, sim.get_particle, 0)
        self.assertRaises(ValueError, sim.get_particle, 1)

        sim.consolidate()
        sim.add_particle()
        sim.add_particle()
        self.assertEqual(sim.get_particle(0).id, 0)
        self.assertEqual(sim.get_particle(1).id, 1)
        self.assertRaises(ValueError, sim.get_particle, 2)
        self.assertRaises(ValueError, sim.get_particle, 3)

    def test_step(self):
        sim = hienoi.dynamics.ParticleSimulation(time_step=0.5)
        sim.add_particle(force=(1.0, 0.0))
        sim.add_particle(force=(2.0, 0.0))
        sim.consolidate()
        p0 = sim.get_particle(0)
        p1 = sim.get_particle(1)
        p2 = sim.add_particle(force=(3.0, 0.0))

        sim.step()
        self.assertEqual(p0.force, (0.0, 0.0))
        self.assertEqual(p1.force, (0.0, 0.0))
        self.assertEqual(p2.force, (3.0, 0.0))
        self.assertEqual(p0.velocity, (0.5, 0.0))
        self.assertEqual(p1.velocity, (1.0, 0.0))
        self.assertEqual(p2.velocity, (0.0, 0.0))
        self.assertEqual(p0.position, (0.25, 0.0))
        self.assertEqual(p1.position, (0.5, 0.0))
        self.assertEqual(p2.position, (0.0, 0.0))

        sim.step()
        self.assertEqual(p0.force, (0.0, 0.0))
        self.assertEqual(p1.force, (0.0, 0.0))
        self.assertEqual(p2.force, (3.0, 0.0))
        self.assertEqual(p0.velocity, (0.5, 0.0))
        self.assertEqual(p1.velocity, (1.0, 0.0))
        self.assertEqual(p2.velocity, (0.0, 0.0))
        self.assertEqual(p0.position, (0.5, 0.0))
        self.assertEqual(p1.position, (1.0, 0.0))
        self.assertEqual(p2.position, (0.0, 0.0))

    def test_consolidate(self):
        sim = hienoi.dynamics.ParticleSimulation()

        sim.consolidate()
        self.assertEqual(len(sim.particles), 0)
        self.assertEqual([particle.id for particle in sim.particles], [])

        p0 = sim.add_particle()
        p1 = sim.add_particle()
        p2 = sim.add_particle()
        p1.alive = False
        sim.consolidate()
        self.assertEqual(len(sim.particles), 2)
        self.assertEqual([particle.id for particle in sim.particles], [0, 2])

        sim.add_particle()
        sim.consolidate()
        self.assertEqual(len(sim.particles), 3)
        self.assertEqual([particle.id for particle in sim.particles], [0, 2, 3])


if __name__ == '__main__':
    from tests.run import run
    run('__main__')



"""Graphical user interface."""

import collections
import ctypes

import sdl2

import hienoi.renderer
from hienoi._common import GLProfile, GraphicsAPI, ParticleDisplay, UserData
from hienoi._vectors import Vector2i, Vector2f, Vector4f


class NavigationAction(object):
    """Enumerator for the current nagivation action.

    Attributes
    ----------
    NONE
    MOVE
    ZOOM
    """

    NONE = 0
    MOVE = 1
    ZOOM = 2


_Handles = collections.namedtuple(
    '_Handles', (
        'window',
        'renderer',
    ))


_GLHandles = collections.namedtuple(
    '_GLHandles', (
        'context',
    ))


_RGBMasks = collections.namedtuple(
    '_RGBMasks', (
        'red',
        'green',
        'blue',
    ))


_FIT_VIEW_REL_PADDING = 2.0

if sdl2.SDL_BYTEORDER == sdl2.SDL_LIL_ENDIAN:
    _RGB_MASKS = _RGBMasks(red=0x000000FF, green=0x0000FF00, blue=0x00FF0000)
else:
    _RGB_MASKS = _RGBMasks(red=0x00FF0000, green=0x0000FF00, blue=0x000000FF)


class GUI(object):
    """GUI.

    Parameters
    ----------
    window_title : str
        Title for the window.
    window_position : hienoi.Vector2i
        Initial window position.
    window_size : hienoi.Vector2i
        Initial window size.
    window_flags : int
        SDL2 window flags.
    view_aperture_x : float
        Initial length in world units to be shown on the X axis.
    view_zoom_range : hienoi.Vector2f
        Zoom value range for the view.
    mouse_wheel_step : float
        Coefficient value for each mouse wheel step.
    grid_density : float
        See :attr:`GUI.grid_density`.
    grid_adaptive_threshold : float
        See :attr:`GUI.grid_adaptive_threshold`.
    show_grid : bool
        See :attr:`GUI.show_grid`.
    background_color : hienoi.Vector4f
        See :attr:`GUI.background_color`.
    grid_color : hienoi.Vector4f
        See :attr:`GUI.grid_color`.
    grid_origin_color : hienoi.Vector4f
        See :attr:`GUI.grid_origin_color`.
    particle_display : int
        See :attr:`GUI.particle_display`.
    point_size : int
        See :attr:`GUI.point_size`.
    edge_feather : float
        See :attr:`GUI.edge_feather`.
    stroke_width : float
        See :attr:`GUI.stroke_width`.
    initialize_callback : function
        Callback function to initialize any GUI state.
        It takes a single argument ``gui``, an instance of this class.
    on_event_callback : function
        Callback function ran during the event polling.
        It takes 3 arguments: ``gui``, an instance of this class,
        ``data``, some data to pass back and forth between the caller and this
        callback function, and ``event``, the event fired.
    renderer : dict
        Keyword arguments for the configuration of the renderer. See the
        parameters for the class :class:`hienoi.renderer.Renderer`.

    Attributes
    ----------
    view_position : hienoi.Vector2f
        Position of the view (camera).
    view_zoom : float
        Current zoom value for the view.
    grid_density : float
        Density of the grid.
        A density of 10.0 means that there are around 10 grid divisions
        displayed on the X axis. A grid division unit represents a fixed length
        in world units, meaning that the actual grid density changes depending
        on the view's zoom.
    show_grid : bool
        True to show the grid.
    background_color : hienoi.Vector4f
        Color for the background.
    grid_color : hienoi.Vector4f
        Color for the grid.
    grid_origin_color : hienoi.Vector4f
        Color for the origin axis of the grid.
    particle_display : int
        Display mode for the particles. Available values are enumerated in the
        :class:`~hienoi.ParticleDisplay` class.
    point_size : int
        Size of the particles in pixels when the display mode is set to
        :attr:`~hienoi.ParticleDisplay.POINT`.
    edge_feather : float
        Feather fall-off in pixels to apply to objects drawn with displays such
        as :attr:`~hienoi.ParticleDisplay.CIRCLE` or
        :attr:`~hienoi.ParticleDisplay.DISC`.
    stroke_width : float
        Width of the stroke in pixels to apply to objects drawn with displays
        such as :attr:`~hienoi.ParticleDisplay.CIRCLE`.
    quit : bool
        ``True`` to signal to the application that it should quit.
    has_view_changed : bool
        ``True`` if the view state has just been changed following an event. It
        is reset to ``False`` whenever :meth:`poll_events` is called.
    user_data : object
        Attribute reserved for any user data.
    """

    def __init__(self,
                 window_title='hienoi',
                 window_position=Vector2i(sdl2.SDL_WINDOWPOS_CENTERED,
                                          sdl2.SDL_WINDOWPOS_CENTERED),
                 window_size=Vector2i(800, 600),
                 window_flags=sdl2.SDL_WINDOW_RESIZABLE,
                 view_aperture_x=100.0,
                 view_zoom_range=Vector2f(1e-6, 1e+6),
                 mouse_wheel_step=0.01,
                 grid_density=10.0,
                 grid_adaptive_threshold=3.0,
                 show_grid=True,
                 background_color=Vector4f(0.15, 0.15, 0.15, 1.0),
                 grid_color=Vector4f(0.85, 0.85, 0.85, 0.05),
                 grid_origin_color=Vector4f(0.85, 0.25, 0.25, 0.25),
                 particle_display=ParticleDisplay.DISC,
                 point_size=4,
                 edge_feather=2.0,
                 stroke_width=0.0,
                 initialize_callback=None,
                 on_event_callback=None,
                 renderer=None):
        renderer = {} if renderer is None else renderer

        if sdl2.SDL_Init(sdl2.SDL_INIT_VIDEO) != 0:
            raise RuntimeError(sdl2.SDL_GetError().decode())

        renderer_info = hienoi.renderer.get_info()
        if renderer_info.api == GraphicsAPI.OPENGL:
            sdl2.SDL_GL_SetAttribute(sdl2.SDL_GL_CONTEXT_MAJOR_VERSION,
                                     renderer_info.major_version)
            sdl2.SDL_GL_SetAttribute(sdl2.SDL_GL_CONTEXT_MINOR_VERSION,
                                     renderer_info.minor_version)
            if renderer_info.profile == GLProfile.CORE:
                sdl2.SDL_GL_SetAttribute(sdl2.SDL_GL_CONTEXT_PROFILE_MASK,
                                         sdl2.SDL_GL_CONTEXT_PROFILE_CORE)

        self._handles = _create_handles(window_title, window_position,
                                        window_size, window_flags,
                                        renderer_info)
        self._renderer = hienoi.renderer.Renderer(**renderer)

        self._initial_view_aperture_x = view_aperture_x
        self._view_zoom_range = view_zoom_range
        self._mouse_wheel_step = mouse_wheel_step
        self._grid_adaptive_threshold = grid_adaptive_threshold
        self._on_event_callback = on_event_callback
        self._listen_for_navigation = False
        self._is_view_manipulated = False

        self.view_position = Vector2f(0.0, 0.0)
        self._view_zoom = 1.0
        self.grid_density = grid_density
        self.show_grid = show_grid
        self.background_color = background_color
        self.grid_color = grid_color
        self.grid_origin_color = grid_origin_color
        self.particle_display = particle_display
        self.point_size = point_size
        self.edge_feather = edge_feather
        self.stroke_width = stroke_width
        self._navigation_action = NavigationAction.NONE
        self.quit = False

        self.user_data = UserData()
        if initialize_callback:
            initialize_callback(self)

    @property
    def view_zoom(self):
        return self._view_zoom

    @view_zoom.setter
    def view_zoom(self, value):
        self._view_zoom = max(self._view_zoom_range[0],
                              min(self._view_zoom_range[1], value))

    @property
    def navigation_action(self):
        return self._navigation_action

    @property
    def has_view_changed(self):
        return self._has_view_changed

    def poll_events(self, scene_state, data=None):
        """Process each event in the queue.

        Parameters
        ----------
        scene_state : hienoi.renderer.SceneState
            Scene state.
        data : object
            Data to pass back and forth between the caller and the function set
            for the 'on event' callback.
        """
        self._has_view_changed = False

        event = sdl2.SDL_Event()
        while sdl2.SDL_PollEvent(ctypes.byref(event)) != 0:
            event_type = event.type
            if event_type == sdl2.SDL_QUIT:
                self._on_quit_event(event.quit)
            elif event_type == sdl2.SDL_WINDOWEVENT:
                self._on_window_event(event.window)
            elif event_type == sdl2.SDL_KEYDOWN:
                self._on_key_down_event(event.key, scene_state)
            elif event_type == sdl2.SDL_KEYUP:
                self._on_key_up_event(event.key)
            elif event_type == sdl2.SDL_MOUSEBUTTONDOWN:
                self._on_mouse_button_down_event(event.button)
            elif event_type == sdl2.SDL_MOUSEBUTTONUP:
                self._on_mouse_button_up_event(event.button)
            elif event_type == sdl2.SDL_MOUSEWHEEL:
                self._on_mouse_wheel_event(event.wheel)
            elif event_type == sdl2.SDL_MOUSEMOTION:
                self._on_mouse_motion_event(event.motion)

            if self._on_event_callback:
                self._on_event_callback(self, data, event)

            if self.quit:
                break

    def render(self, scene_state):
        """Render a new frame.

        Parameters
        ----------
        scene_state : hienoi.renderer.SceneState
            Scene state.
        """
        renderer_state = hienoi.renderer.State(
            window_size=self.get_window_size(),
            view_position=self.view_position,
            view_zoom=self._view_zoom,
            origin=self.world_to_screen(Vector2f(0.0, 0.0)),
            initial_view_aperture_x=self._initial_view_aperture_x,
            view_aperture=self.get_view_aperture(),
            grid_density=self.grid_density,
            grid_adaptive_threshold=self._grid_adaptive_threshold,
            background_color=self.background_color,
            grid_color=self.grid_color,
            grid_origin_color=self.grid_origin_color,
            show_grid=self.show_grid,
            particle_display=self.particle_display,
            point_size=self.point_size,
            edge_feather=self.edge_feather,
            stroke_width=self.stroke_width,
        )

        self._renderer.render(renderer_state, scene_state)

        if hienoi.renderer.get_info().api == GraphicsAPI.OPENGL:
            sdl2.SDL_GL_SwapWindow(self._handles.window)

    def terminate(self):
        """Cleanup the GUI resources."""
        self._renderer.cleanup()
        if hienoi.renderer.get_info().api == GraphicsAPI.OPENGL:
            sdl2.SDL_GL_DeleteContext(self._handles.renderer.context)

        sdl2.SDL_DestroyWindow(self._handles.window)
        sdl2.SDL_Quit()

    def get_window_size(self):
        """Retrieve the window size.

        Returns
        -------
        hienoi.Vector2i
            The window size.
        """
        window_size_x = ctypes.c_int()
        window_size_y = ctypes.c_int()
        sdl2.SDL_GetWindowSize(self._handles.window,
                               ctypes.byref(window_size_x),
                               ctypes.byref(window_size_y))
        return Vector2i(window_size_x.value, window_size_y.value)

    def get_view_aperture(self):
        """Retrieve the view aperture.

        It represents the area in world units covered by the view.

        Returns
        -------
        hienoi.Vector2f
            The view aperture.
        """
        window_size = self.get_window_size()
        aperture_x = self._initial_view_aperture_x / self._view_zoom
        return Vector2f(aperture_x, aperture_x * window_size.y / window_size.x)

    def get_mouse_position(self):
        """Retrieve the mouse position in screen space.

        Returns
        -------
        hienoi.Vector2i
            The mouse position.
        """
        position_x = ctypes.c_int()
        position_y = ctypes.c_int()
        sdl2.SDL_GetMouseState(ctypes.byref(position_x),
                               ctypes.byref(position_y))
        return Vector2i(position_x.value, position_y.value)

    def get_screen_to_world_ratio(self):
        """Retrieve the ratio to convert a sreen unit into a world unit.

        Returns
        -------
        float
            The screen to world ratio.
        """
        window_size = self.get_window_size()
        aperture_x = self._initial_view_aperture_x / self._view_zoom
        return aperture_x / window_size.x

    def screen_to_world(self, point):
        """Convert a point from screen space to world space coordinates.

        Parameters
        ----------
        point : hienoi.Vector2i
            Point in screen space coordinates.

        Returns
        -------
        hienoi.Vector2f
            The point in world space coordinates.
        """
        window_size = self.get_window_size()
        view_aperture = self.get_view_aperture()
        return Vector2f(
            (self.view_position.x
             + (point.x - window_size.x / 2.0)
             * view_aperture.x / window_size.x),
            (self.view_position.y
             - (point.y - window_size.y / 2.0)
             * view_aperture.y / window_size.y))

    def world_to_screen(self, point):
        """Convert a point from world space to screen space coordinates.

        Parameters
        ----------
        point : hienoi.Vector2f
            Point in world space coordinates.

        Returns
        -------
        hienoi.Vector2i
            The point in screen space coordinates.
        """
        window_size = self.get_window_size()
        view_aperture = self.get_view_aperture()
        return Vector2i(
            int(round(
                (window_size.x / view_aperture.x)
                * (-self.view_position.x + point.x + view_aperture.x / 2.0))),
            int(round(
                (window_size.y / view_aperture.y)
                * (self.view_position.y - point.y + view_aperture.y / 2.0))))

    def write_snapshot(self, filename):
        """Take a snapshot of the view and write it as a BMP image.

        Parameters
        ----------
        filename : str
            Destination filename.
        """
        pixel_size = 4
        pixels = self._renderer.read_pixels()
        surface = sdl2.SDL_CreateRGBSurfaceFrom(
            pixels.data, pixels.width, pixels.height,
            8 * pixel_size, pixels.width * pixel_size,
            _RGB_MASKS.red, _RGB_MASKS.green, _RGB_MASKS.blue, 0)
        sdl2.SDL_SaveBMP(surface, filename)
        sdl2.SDL_FreeSurface(surface)

    def _reset_view(self):
        """Reset the view position and zoom."""
        self.view_position = Vector2f(0.0, 0.0)
        self.view_zoom = 1.0
        self._has_view_changed = True

    def _fit_view(self, scene_state):
        """Fit the view to the scene."""
        if len(scene_state.particles) > 1:
            window_size = self.get_window_size()
            initial_size = Vector2f(
                self._initial_view_aperture_x,
                self._initial_view_aperture_x * window_size.y / window_size.x)

            lower_bounds = scene_state.lower_bounds
            upper_bounds = scene_state.upper_bounds
            required_size = (upper_bounds - lower_bounds).iscale(
                _FIT_VIEW_REL_PADDING)
            required_size = Vector2f(
                max(required_size.x,
                    initial_size.x * self._view_zoom_range[0]),
                max(required_size.y,
                    initial_size.y * self._view_zoom_range[0]))

            self.view_position = (lower_bounds + upper_bounds).iscale(0.5)
            self.view_zoom = min(initial_size.x / required_size.x,
                                 initial_size.y / required_size.y)
        elif len(scene_state.particles) == 1:
            self.view_position = Vector2f(
                *scene_state.particles['position'][0])
            self.view_zoom = 1.0
        else:
            self._reset_view()

        self._has_view_changed = True

    def _on_quit_event(self, event):
        """Event 'on quit'."""
        self.quit = True

    def _on_window_event(self, event):
        """Event 'on window'."""
        if event.event == sdl2.SDL_WINDOWEVENT_SIZE_CHANGED:
            self._renderer.resize(event.data1, event.data2)

    def _on_key_down_event(self, event, scene_state):
        """Event 'on key down'."""
        code = event.keysym.sym
        modifier = event.keysym.mod
        if modifier == sdl2.KMOD_NONE:
            if code == sdl2.SDLK_SPACE:
                self._listen_for_navigation = True
            elif code == sdl2.SDLK_d:
                self.particle_display = (
                    (self.particle_display + 1) % (ParticleDisplay._LAST + 1))
            elif code == sdl2.SDLK_f:
                self._fit_view(scene_state)
            elif code == sdl2.SDLK_g:
                self.show_grid = not self.show_grid
            elif code == sdl2.SDLK_r:
                self._reset_view()

    def _on_key_up_event(self, event):
        """Event 'on key up'."""
        code = event.keysym.sym
        if code == sdl2.SDLK_SPACE:
            self._listen_for_navigation = False

    def _on_mouse_button_down_event(self, event):
        """Event 'on mouse button down'."""
        if self._listen_for_navigation:
            if event.button == sdl2.SDL_BUTTON_LEFT:
                self._navigation_action = NavigationAction.MOVE
            elif event.button == sdl2.SDL_BUTTON_RIGHT:
                self._navigation_action = NavigationAction.ZOOM

    def _on_mouse_button_up_event(self, event):
        """Event 'on mouse button up'."""
        if (event.button == sdl2.SDL_BUTTON_LEFT
                or event.button == sdl2.SDL_BUTTON_RIGHT):
            self._navigation_action = NavigationAction.NONE

    def _on_mouse_wheel_event(self, event):
        """Event 'on mouse wheel'."""
        scale = 1.0 + self._mouse_wheel_step * event.y
        self.view_zoom *= scale
        self._has_view_changed = True

    def _on_mouse_motion_event(self, event):
        """Event 'on mouse motion'."""
        window_size = self.get_window_size()
        view_aperture = self.get_view_aperture()
        if self._navigation_action == NavigationAction.MOVE:
            self.view_position.set(
                (self.view_position.x
                 - event.xrel * view_aperture.x / window_size.x),
                (self.view_position.y
                 + event.yrel * view_aperture.y / window_size.y))
            self._has_view_changed = True
        elif self._navigation_action == NavigationAction.ZOOM:
            scale = (1.0
                     + float(event.xrel) / window_size.x
                     - float(event.yrel) / window_size.y)
            self.view_zoom *= scale
            self._has_view_changed = True


def _create_handles(window_title, window_position, window_size, window_flags,
                    renderer_info):
    """Create the SDL2 handles."""
    window_flags = sdl2.SDL_WINDOW_SHOWN | window_flags
    if renderer_info.api == GraphicsAPI.OPENGL:
        window_flags |= sdl2.SDL_WINDOW_OPENGL
        window = sdl2.SDL_CreateWindow(
            window_title.encode(),
            window_position.x, window_position.y,
            window_size.x, window_size.y,
            window_flags)
        if not window:
            raise RuntimeError(sdl2.SDL_GetError().decode())

        context = sdl2.SDL_GL_CreateContext(window)
        if not context:
            raise RuntimeError(sdl2.SDL_GetError().decode())

        # Try to disable the vertical synchronization. It applies to the active
        # context and thus needs to be called after `SDL_GL_CreateContext`.
        sdl2.SDL_GL_SetSwapInterval(0)

        return _Handles(
            window=window,
            renderer=_GLHandles(context=context))



"""Glue for the different parts of the application.

Gets the modules running, connects them together through inter-process
messaging, and orchestrates the whole.
"""

import bisect
import collections
import cProfile
import logging
import multiprocessing
import os
import pstats
import sys
import tempfile
import timeit
import traceback

import hienoi._common
import hienoi.dynamics
import hienoi.gui
import hienoi.renderer
from hienoi._dynamicarray import DynamicArray


if sys.version_info[0] == 2:
    import Queue as queue

    def _iteritems(d, **kwargs):
        return d.iteritems(**kwargs)

    _range = xrange
else:
    import queue

    def _iteritems(d, **kwargs):
        return iter(d.items(**kwargs))

    _range = range


_clock = timeit.default_timer


class LoggingLevel(object):
    """Enumerator for the logging levels.

    Attributes
    ----------
    CRITICAL
    ERROR
    WARNING
    INFO
    DEBUG
    NOTSET
    """

    CRITICAL = logging.CRITICAL
    ERROR = logging.ERROR
    WARNING = logging.WARNING
    INFO = logging.INFO
    DEBUG = logging.DEBUG
    NOTSET = logging.NOTSET


class Status(object):
    """Enumerator for the return status codes.

    Attributes
    ----------
    Success
    Failure
    """

    SUCCESS = 0
    FAILURE = 1


_Configs = collections.namedtuple(
    '_Configs', (
        'application',
        'gui',
        'particle_simulation',
    ))


_InitialSimulationStates = collections.namedtuple(
    '_InitialSimulationStates', (
        'particle',
    ))


_Buffer = collections.namedtuple(
    '_Buffer', (
        'data',
        'metadata',
    ))


_BufferMetadata = collections.namedtuple(
    '_BufferMetadata', (
        'dtype',
    ))


_OnEventData = collections.namedtuple(
    'OnEventData', (
        'callbacks',
    ))


class OnEventData(_OnEventData):
    """Data to pass to the GUI's 'on event' callback.

    Attributes
    ----------
    callbacks : hienoi.application.Callbacks
        Inter-process callbacks.
    """

    __slots__ = ()


_Callbacks = collections.namedtuple(
    'Callbacks', (
        'particle_simulation',
    ))


class Callbacks(_Callbacks):
    """Inter-process callbacks.

    Attributes
    ----------
    particle_simulation : sequence of hienoi.application.Callback
        Callbacks to run just before the method
        :meth:`hienoi.dynamics.ParticleSimulation.step` is called.
    """

    __slots__ = ()

    def __new__(cls):
        base = super(Callbacks, cls)
        return base.__new__(cls, *([] for _ in base._fields))


_Callback = collections.namedtuple(
    'Callback', (
        'function',
        'args',
        'kwargs',
    ))
_Callback.__new__.__defaults__ = (
    None,  # args
    None,  # kwargs
)


class Callback(_Callback):
    """Inter-process callback.

    Attributes
    ----------
    function : function
        Callback function. It needs to declare ``obj``, the object specific to
        the target process' module, as first parameter. Additional parameters
        declared are passed the ``args`` and ``kwargs`` values.
    args : tuple
        Additional positional arguments to pass to the function.
    kwargs : dict
        Additional keyword arguments to pass to the function.
    """

    __slots__ = ()


_Process = collections.namedtuple(
    '_Process', (
        'instance',
        'output_queues',
        'output_pipes',
    ))


# Enumerator for the inter-process messages.
_MESSAGE_ERROR = 0
_MESSAGE_STOP = 1
_MESSAGE_QUIT = 2
_MESSAGE_REQUEST_SIM_UPDATE = 3
_MESSAGE_SIM_UPDATE = 4
_MESSAGE_RUN_CALLBACKS = 5
_MESSAGE__LAST = _MESSAGE_RUN_CALLBACKS


_Message = collections.namedtuple(
    '_Message', (
        'type',
    ))


_ErrorMessage = collections.namedtuple(
    '_ErrorMessage', (
        'type',
        'process_id',
        'process_name',
        'message',
    ))


_SimUpdateMessage = collections.namedtuple(
    '_SimUpdateMessage', (
        'type',
        'time',
    ))


_RunCallbacksMessage = collections.namedtuple(
    '_RunCallbacksMessage', (
        'type',
        'callbacks',
    ))


_MESSAGE_STRUCTS = {
    _MESSAGE_ERROR: _ErrorMessage,
    _MESSAGE_STOP: _Message,
    _MESSAGE_QUIT: _Message,
    _MESSAGE_REQUEST_SIM_UPDATE: _Message,
    _MESSAGE_SIM_UPDATE: _SimUpdateMessage,
    _MESSAGE_RUN_CALLBACKS: _RunCallbacksMessage,
}
assert len(_MESSAGE_STRUCTS) == _MESSAGE__LAST + 1


_Time = collections.namedtuple(
    '_Time', (
        'value',
        'unit',
    ))


def run(application=None, gui=None, particle_simulation=None):
    """Main entry point to get the application running.

    Parameters
    ----------
    application: dict
        Keyword arguments for the configuration of the application:

        - logging_level : :class:`LoggingLevel`
            Logging messages having a lower threshold than the given level are
            ignored.
        - profile : bool
            ``True`` to enable a profiler to monitor the performances of the
            application. Each process is reported separately.
        - time_step : float
            Amount by which all the simulation times are being incremented by
            at each step. The lower the value, the more accurate the
            simulations are, but the more compute-intensive they become. This
            global time step overrides any time step set in the parameter
            ``particle_simulation``.
        - capture : bool
            ``True`` to write a ``.bmp`` snapshot for each frame to the
            destination pointed by ``capture_filename``.
        - capture_filename : str
            Filename of the snapshots to write. The frame number can be defined
            through Python's formatting syntax. For example, the pattern
            ``/destination/directory/{frame:04}.bmp`` outputs filenames with 4
            digits padded with zeroes, such as ``0009.bmp``, ``0085.bmp``, and
            so on. If ``None``, the snapshots are outputted in a temporary
            folder.
        - capture_increment : int
            Capture a snapshot every n-th frame.
    gui : dict
        Keyword arguments for the configuration of the GUI. See the parameters
        for the class :class:`hienoi.gui.GUI`.
    particle_simulation : dict
        Keyword arguments for the configuration of the particle simulation. See
        the parameters for the class
        :class:`hienoi.dynamics.ParticleSimulation`.

    Returns
    -------
    int
        A value of :attr:`Status.SUCCESS` if successful, or
        :attr:`Status.FAILURE` otherwise.
    """
    configs = {
        'application': {
            'logging_level': LoggingLevel.NOTSET,
            'profile': False,
            'time_step': 0.02,
            'capture': False,
            'capture_filename': None,
            'capture_increment': 1,
        },
        'gui': gui,
        'particle_simulation': particle_simulation,
    }
    configs = {key: {} if value is None else value
               for key, value in _iteritems(configs)}
    configs['application'].update({} if application is None else application)
    configs['particle_simulation']['time_step'] = (
        configs['application']['time_step'])
    configs = _Configs(**configs)

    if configs.application['profile']:
        return _run_profiler(_run, configs)

    return _run(configs)


def _run(configs):
    """Implementation for the `run()` function."""
    logger = multiprocessing.log_to_stderr()
    logger.setLevel(configs.application['logging_level'])

    # Messages from the main process to the child ones.
    downwards_queue = multiprocessing.Queue()
    # Messages from the child processes to the main one.
    upwards_queue = multiprocessing.Queue()
    # Messages from the GUI process to the particle simulation one.
    gui_to_psim_pipe = multiprocessing.Pipe(duplex=False)
    # Messages from the particle simulation process to the GUI one.
    psim_to_gui_pipe = multiprocessing.Pipe(duplex=False)

    if (configs.application['capture']
            and configs.application['capture_filename'] is None):
        directory = tempfile.mkdtemp()
        logger.info("Created a temporary directory for the snapshots: %s"
                    % (directory,))
        configs.application['capture_filename'] = os.path.join(
            directory, '{frame:04}.bmp')

    initial_sim_states = _InitialSimulationStates(
        particle=hienoi.dynamics.ParticleSimulation(
            **configs.particle_simulation))

    # The application is spawning child processes which comes with an overhead
    # mostly in the form of extra copies of data states needing to be made and
    # inter-process messaging latency, which may or may not impact the
    # execution speed. Since Hienoi aims at being of educational value,
    # performances implications have been disregarded here in favour of
    # demonstrating a multiprocessing approach.
    processes = (
        _Process(
            instance=_create_process(
                _gui_process, 'gui',
                args=(gui_to_psim_pipe[1], upwards_queue, psim_to_gui_pipe[0],
                      downwards_queue, initial_sim_states, configs),
                kwargs={},
                upwards=upwards_queue,
                profile=configs.application['profile']),
            output_queues=(upwards_queue,),
            output_pipes=(gui_to_psim_pipe[0],)),
        _Process(
            instance=_create_process(
                _particle_simulation_process, 'particle_simulation',
                args=(psim_to_gui_pipe[1], gui_to_psim_pipe[0],
                      downwards_queue, initial_sim_states, configs),
                kwargs={},
                upwards=upwards_queue,
                profile=configs.application['profile']),
            output_queues=(upwards_queue,),
            output_pipes=(psim_to_gui_pipe[0],)),
    )

    try:
        for process in processes:
            process.instance.start()

        return_code = _main_process(logger, downwards_queue, upwards_queue,
                                    len(processes), configs)
    finally:
        for process in processes:
            while process.instance.is_alive():
                # Any data pushed into the queues by a child process needs
                # to be flushed before being able to join that process.
                for q in process.output_queues:
                    _flush_queue(q)

                # Sometimes pipes also need to be flushed to unblock a process
                # that started sending a large chunk a data to another process
                # that has stopped in the meantime.
                for c in process.output_pipes:
                    _flush_pipe(c)

            if process.instance.pid != None:
                process.instance.join()

    return return_code


def _main_process(logger, downwards, upwards, process_count, configs):
    """Main process."""
    try:
        while True:
            message = _receive_message(upwards, block=True)
            if message.type == _MESSAGE_ERROR:
                logger.error("Process '%s' [%d]:\n%s" % (
                    message.process_name, message.process_id, message.message))
                return Status.FAILURE
            elif message.type == _MESSAGE_QUIT:
                break
    finally:
        for _ in _range(process_count):
            _send_message(downwards, _MESSAGE_STOP)

        downwards.close()

    return Status.SUCCESS


def _gui_process(to_psim, upwards, from_psim, downwards, sims, configs):
    """GUI process."""
    gui = None
    try:
        gui = hienoi.gui.GUI(**configs.gui)
        time_step = configs.application['time_step']
        capture = configs.application['capture']
        capture_filename = configs.application['capture_filename']
        capture_increment = configs.application['capture_increment']

        current_state = hienoi.renderer.SceneState(
            time=0.0,
            particles=sims.particle.particles.data)
        del sims

        # Because the initial simulation state is already available, the first
        # partial state is to be assembled at the next time step.
        partial_state = {'time': time_step}
        next_state = None

        request_sim_updates = True
        frame = 0
        render = True
        accumulator = 0.0
        previous_time = 0.0
        elapsed_time = 0.0
        time_start = _clock()
        while True:
            message = _receive_message(downwards)
            if message is None:
                pass
            elif message.type == _MESSAGE_STOP:
                break

            on_event_data = OnEventData(callbacks=Callbacks())
            gui.poll_events(current_state, data=on_event_data)
            if gui.quit:
                _send_message(upwards, _MESSAGE_QUIT)
                break

            if on_event_data.callbacks.particle_simulation:
                _send_message(
                    to_psim, _MESSAGE_RUN_CALLBACKS,
                    callbacks=on_event_data.callbacks.particle_simulation)

            if next_state is None:
                if request_sim_updates:
                    _send_message(to_psim, _MESSAGE_REQUEST_SIM_UPDATE)
                    request_sim_updates = False

                if 'particles' not in partial_state:
                    message = _receive_message(from_psim)
                    if message is None:
                        pass
                    elif message.type == _MESSAGE_SIM_UPDATE:
                        assert (abs(message.time - partial_state['time'])
                                < time_step * 0.1)
                        buf = _receive_buffer(from_psim)
                        particles = _buffer_to_array(buf)
                        partial_state['particles'] = particles.data

            if _is_scene_state_complete(partial_state):
                next_state = hienoi.renderer.SceneState(**partial_state)
                request_sim_updates = True

            accumulator += elapsed_time
            if accumulator >= time_step and next_state is not None:
                current_state = next_state
                partial_state = {'time': current_state.time + time_step}
                next_state = None
                accumulator = 0.0
                render = True

            if render:
                gui.render(current_state)
                if capture and frame % capture_increment == 0:
                    filename = capture_filename.format(frame=frame)
                    gui.write_snapshot(filename)

                frame += 1
                render = False

            current_time = _clock() - time_start
            elapsed_time = current_time - previous_time
            previous_time = current_time
    finally:
        to_psim.close()
        if gui is not None:
            gui.terminate()


def _particle_simulation_process(to_gui, from_gui, downwards, sims, configs):
    """Particle simulation process."""
    try:
        sim = sims.particle
        particles = DynamicArray(0, hienoi._common.PARTICLE_NANI.dtype)
        callbacks = None

        # Step the simulation without sending the initial state over to the
        # GUI process since it already has initialized its own copy.
        step = True
        send = False
        while True:
            message = _receive_message(downwards)
            if message is None:
                pass
            elif message.type == _MESSAGE_STOP:
                break

            while True:
                message = _receive_message(from_gui)
                if message is None:
                    break
                elif message.type == _MESSAGE_REQUEST_SIM_UPDATE:
                    send = True
                elif message.type == _MESSAGE_RUN_CALLBACKS:
                    callbacks = message.callbacks

            if step:
                if callbacks is not None:
                    _run_callbacks(sim, callbacks)
                    sim.consolidate()
                    callbacks = None

                sim.step()
                step = False

            if send:
                _send_message(to_gui, _MESSAGE_SIM_UPDATE, time=sim.time)

                # Copy only the relevant rendering attributes of the simulation
                # state into a contiguous array.
                particles.copy_from(sim.particles.data)

                _send_buffer(to_gui, _array_to_buffer(particles))
                step = True
                send = False
    finally:
        to_gui.close()


def _run_profiler(function, *args, **kwargs):
    """Run a profiler on the specified function."""
    profiler = cProfile.Profile()
    profiler.enable()
    result = function(*args, **kwargs)
    profiler.disable()
    stats = pstats.Stats(profiler).sort_stats('cumtime')
    stats.print_stats()
    return result


def _create_process(target, name, args, kwargs, upwards, profile):
    """Create a new process."""
    process = multiprocessing.Process(
        target=_process_wrapper,
        name=name,
        args=(target, upwards, profile) + args,
        kwargs=kwargs)
    process.daemon = False
    return process


def _process_wrapper(function, upwards, profile, *args, **kwargs):
    """Wrap a process with additional features."""
    try:
        if profile:
            _run_profiler(function, *args, **kwargs)
        else:
            function(*args, **kwargs)
    except Exception:
        process = multiprocessing.current_process()
        info = sys.exc_info()
        exception = traceback.format_exception(
            info[0], info[1], info[2].tb_next)
        _send_message(upwards, _MESSAGE_ERROR,
                      process_id=process.pid,
                      process_name=process.name,
                      message=''.join(exception).rstrip())
    finally:
        upwards.close()


def _is_scene_state_complete(state):
    """Check if a scene state dictionary has a complete definition."""
    return all(field in state for field in hienoi.renderer.SceneState._fields)


def _run_callbacks(obj, callbacks):
    """Run a sequence of inter-process callbacks."""
    for callback in callbacks:
        args = () if callback.args is None else callback.args
        kwargs = {} if callback.kwargs is None else callback.kwargs
        callback.function(obj, *args, **kwargs)


def _buffer_to_array(buf):
    """Unpack a buffer object into an array."""
    return DynamicArray.from_buffer(buf.data, buf.metadata.dtype)


def _array_to_buffer(array):
    """Pack an array into a buffer object."""
    return _Buffer(data=array.data,
                   metadata=_BufferMetadata(dtype=array.dtype))


def _send_message(c, type, **kwargs):
    """Send a message."""
    message = _MESSAGE_STRUCTS[type](type=type, **kwargs)
    if isinstance(c, multiprocessing.queues.Queue):
        c.put(message, block=True)
    else:
        c.send(message)


def _receive_message(c, block=False):
    """Receive a message."""
    if isinstance(c, multiprocessing.queues.Queue):
        try:
            message = c.get(block=block)
        except queue.Empty:
            return None
    else:
        if not block and not c.poll():
            return None

        try:
            message = c.recv()
        except EOFError:
            return None

    return message


def _send_buffer(c, buf):
    """Send a buffer object to a pipe connection."""
    c.send(buf.metadata)
    c.send_bytes(buf.data)


def _receive_buffer(c):
    """Receive a buffer object from a pipe connection."""
    metadata = c.recv()
    data = c.recv_bytes()
    return _Buffer(
        data=data,
        metadata=metadata)


def _flush_queue(q):
    """Flush the content of a queue."""
    try:
        while True:
            q.get(block=False)
    except queue.Empty:
        pass


def _flush_pipe(c):
    """Flush the content of a pipe."""
    try:
        while c.poll():
            c.recv_bytes()
    except EOFError:
        pass


def _pick_time_unit(value):
    """Pick the most readable time unit."""
    bounds = (1e-9, 1e-6, 1e-3)
    units = 'num'
    if value >= 1.0:
        out = _Time(value=value, unit='s')
    elif value <= bounds[0] * 1e-3:
        out = _Time(value=0.0, unit='%ss' % (units[0],))
    else:
        i = max(0, bisect.bisect(bounds, value) - 1)
        out = _Time(value=value / bounds[i], unit='%ss' % (units[i],))

    return out



#!/usr/bin/env python

"""Introductory demo showing a basic set-up.

Features:

- simulation callbacks: spawn a single particle then, at each simulation
  step, add a force towards the origin proportional to its distance to it.
"""

import os
import sys

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

import hienoi.application
from hienoi import Vector2f


def initialize_particle_simulation(sim):
    """Callback to initialize the particle simulation state.

    Parameters
    ----------
    sim : hienoi.dynamics.ParticleSimulation
        Particle simulation.
    """
    # Let's add a single particle to the simulation. Since in the `run()`
    # function below we override the applications's configuration attribute
    # `view_aperture` with a value of 80.0, then the view is set to initially
    # show 80 world units in the X axis. By setting the initial X position of
    # the particle to 30.0, it should spawn at the 3/4th location of the
    # screen's right half.
    sim.add_particle(position=Vector2f(30.0, 0.0))


def update_particle_simulation(sim):
    """Callback to update the particle simulation state.

    Parameters
    ----------
    sim : hienoi.dynamics.ParticleSimulation
        Particle simulation.
    """
    # Because this function is passed to the particle simulation's
    # `postsolve_callback`, it runs at the end of each simulation step. As a
    # result, the changes applied here are rendered as-is until the next
    # simulation step. If we'd want to have our changes to the force and
    # velocity attributes to be immediately solved by the integrator then we'd
    # use `presolve_callback` instead.

    # Now let's retrieve the particle created during initialization and apply
    # an attractive force towards the origin to it. But before doing so,
    # reading some explanations provided with the class
    # `hienoi.dynamics.ParticleSimulation` might prove useful.

    # To attract the particle to the origin, we need a force, defined as a 2D
    # vector, that starts at the current particle's position and that is
    # directed towards the origin. An example of such a force is the vector
    # (origin.x - position.x, origin.y - position.y), which can be simplified
    # to (-position.x, -position.y) since the origin has coordinates (0, 0).
    # The vector could be further scaled to increase or decrease its magnitude,
    # thus influencing how fast the particle moves towards its goal.
    particle = sim.particles[0]
    particle.force -= particle.position


def run():
    """Run the application."""
    # Each module in Hienoi provides a configuration object that we can use to
    # hook our code into the framework but also to override default settings
    # such as the simulation's time step.
    return hienoi.application.run(
        gui={
            'window_title': 'intro',
            'view_aperture_x': 80.0,
            'grid_density': 8.0,
        },
        particle_simulation={
            'time_step': 0.01,
            'initialize_callback': initialize_particle_simulation,
            'postsolve_callback': update_particle_simulation,
        })


if __name__ == '__main__':
    sys.exit(run())



"""Physics systems evolving over time."""

import itertools
import math
import sys

import numpy

import hienoi._common
import hienoi._numeric
from hienoi._common import UserData
from hienoi._dynamicarray import DynamicArray
from hienoi._kdtree import KDTree
from hienoi._nani import Bool, Number, READ_ONLY, PickableNaniStructure
from hienoi._numeric import Int32, Float32
from hienoi._orderedbuffer import OrderedBuffer
from hienoi._vectors import VECTOR2F


if sys.version_info[0] == 2:
    def _iteritems(d, **kwargs):
        return d.iteritems(**kwargs)

    _zip = itertools.izip
else:
    def _iteritems(d, **kwargs):
        return iter(d.items(**kwargs))

    _zip = zip


class ParticleSimulation(object):
    """Particle simulation.

    Such a system allows to have particles moving over time based on basic laws
    of motion.

    Each particle have attributes such as position, velocity, force, and mass.
    What the force attribute really represents is the acceleration force. When
    running the simulation, at each time interval `t` defined by the
    simulation's time step, the simulation is stepped, that is, its current
    state is solved to advance to the next state corresponding to a later point
    in time `t + time_step`.

    The solver's operation consists in integrating the [acceleration] force of
    each particle to retrieve their new velocity, and in turn integrating this
    velocity to obtain their new position. The particles end up moving over
    time according to the forces applied to them.

    Particles can be added, removed, and can have their attributes modified at
    any point in time through three hooks provided: ``initialize_callback``,
    ``presolve_callback``, and ``postsolve_callback``.

    Warning
    -------
    Any modification to the particle attributes is applied in-place, that is
    with immediate effect. However the addition and removal of particles is
    only really executed at a later time.

    This behaviour is noticable when querying the simulation's state: a
    particle just added isn't taken into account (it lives in a buffer), while
    one that has just been marked as 'not alive' is still queried.

    Another way to reason about this is to think of each callback as being part
    of a node graph. A callback is like a node which takes an upstream
    simulation state as input and outputs a new simulation state downstream.
    Whenever querying is required, it is being done on the set of particles
    available in the upstream state, and the downstream state is only written
    once the callback has finished evaluating.

    The method :meth:`consolidate` is the one responsible for processing all
    the additions and removals requested and is automatically run after
    each callback. It can also be called manually within the callbacks but it
    isn't a recommended approach since it comes with a drawbacks, see
    :meth:`consolidate`.

    Parameters
    ----------
    time_step : float
        See :attr:`ParticleSimulation.time_step`.
    initial_particle_capacity : int
        Initial number of elements allocated for the particle array.
    particle_bucket_buffer_capacity : int
        New particles cannot always be directly added to the array and
        therefore temporarily end up in a buffer until the function
        :func:`consolidate` is run. The given bucket buffer capacity
        represents the number of elements that a single bucket can hold.
    particle_attributes : sequence of hienoi.Field or compatible tuple
        Additional attributes to define for each particle.
    initialize_callback : function
        Callback function to initialize the simulation.
        It takes a single argument ``sim``, an instance of this class.
    presolve_callback : function
        Callback function executed before solving the simulation.
        It takes a single argument ``sim``, an instance of this class.
    postsolve_callback : function
        Callback function executed after solving the simulation.
        It takes a single argument ``sim``, an instance of this class.

    Attributes
    ----------
    time_step : float
        Amount by which the simulation time is being incremented by at each
        step. The lower the value, the more accurate the simulation is, but the
        more compute-intensive it becomes.
    time : float
        Simulation time.
    last_particle_id : int
        ID of the last particle created.
    particles : sequence of nani.Particle
        All the particles.
    user_data : object
        Attribute reserved for any user data.
    """

    _ATTRS = (
        ('id', Number(type=Int32, default=-1), READ_ONLY),
        ('alive', Bool(default=True)),
        ('position', hienoi._common.PARTICLE_ATTRS.position.nani),
        ('velocity', VECTOR2F),
        ('force', VECTOR2F),
        ('mass', Number(type=Float32, default=1.0)),
        ('size', hienoi._common.PARTICLE_ATTRS.size.nani),
        ('color', hienoi._common.PARTICLE_ATTRS.color.nani),
    )
    _ATTR_ID_NUMPY_TYPE = hienoi._numeric.to_numpy(_ATTRS[0][1].type)

    class Neighbours(object):
        """Neighbour sequence."""

        def __init__(self, neighbours, particles, particle_view):
            self._neighbours = neighbours
            self._particles = particles
            self._particle_view = particle_view

        def __len__(self):
            return len(self._neighbours)

        def __iter__(self):
            return (
                ParticleSimulation.Neighbour(
                    item, self._particle_view(self._particles[item['index']]))
                for item in self._neighbours)

        @property
        def data(self):
            return self._neighbours

    class Neighbour(object):
        """Neighbour object."""

        def __init__(self, data, particle):
            self._data = data
            self._particle = particle

        @property
        def particle(self):
            return self._particle

        @property
        def squared_distance(self):
            return self._data['squared_distance']

        @property
        def distance(self):
            return math.sqrt(self._data['squared_distance'])

    def __init__(self,
                 time_step=0.02,
                 initial_particle_capacity=512,
                 particle_bucket_buffer_capacity=256,
                 particle_attributes=None,
                 initialize_callback=None,
                 presolve_callback=None,
                 postsolve_callback=None):
        attrs = self._ATTRS
        if particle_attributes is not None:
            attrs += particle_attributes

        self._nani = PickableNaniStructure(attrs, 'Particle')

        self._presolve_callback = presolve_callback
        self._postsolve_callback = postsolve_callback

        self._time_step = time_step
        self._time = 0.0
        self._last_id = -1

        # Particles are stored in a contiguous, resizable, array.
        self._array = DynamicArray(initial_particle_capacity, self._nani.dtype)

        # New particles are always added into a temporary buffer until
        # consolidation.
        self._buffer = OrderedBuffer(particle_bucket_buffer_capacity,
                                     self._nani.dtype)

        self._kd_tree = None

        self.user_data = UserData()
        if initialize_callback:
            initialize_callback(self)
            self.consolidate()

    @property
    def time_step(self):
        return self._time_step

    @property
    def time(self):
        return self._time

    @property
    def last_particle_id(self):
        return self._last_id

    @property
    def particles(self):
        return self._nani.view(self._array.data)

    def add_particle(self, **kwargs):
        """Add a new particle.

        Any read-only attribute passed to the ``kwargs`` parameter, such as the
        'id' attribute, are discarded.

        Parameters
        ----------
        kwargs
            Keyword arguments to override the default attribute values.

        Returns
        -------
        nani.Particle
            The new particle.
        """
        kwargs = kwargs.copy()
        kwargs['id'] = self._last_id + 1
        particle = self._buffer.append(self._nani.default._replace(**kwargs))
        self._last_id = particle['id']
        return self._nani.element_view(particle)

    def add_particles(self, count):
        """Add a bunch of new particles.

        Parameters
        ----------
        count : int
            Number of particles to add.

        Returns
        -------
        sequence of nani.Particle
            The new particles.
        """
        array = numpy.empty(count, dtype=self._nani.dtype)
        array[:] = self._nani.default
        array['id'] = numpy.arange(self._last_id + 1,
                                   self._last_id + 1 + count)
        particles = self._buffer.extend(array)
        self._last_id = particles['id'][-1]
        return self._nani.view(particles)

    def get_particle(self, id):
        """Retrieve a particle.

        Parameters
        ----------
        id : int
            ID of the particle to retrieve.

        Returns
        -------
        nani.Particle
            The particle found.
        """
        # PRECONDITION: `self._array.data` sorted by id.
        id = self._ATTR_ID_NUMPY_TYPE(id)
        idx = numpy.searchsorted(self._array.data['id'], id)
        if idx < len(self._array) and self._array.data[idx]['id'] == id:
            return self._nani.element_view(self._array.data[idx])

        raise ValueError("No particle found with ID '%d'." % (id,))

    def get_neighbour_particles(self, point, count=1, radius=None, sort=False):
        """Retrieve the particles neighbour to a point."""
        if self._kd_tree is None:
            self._kd_tree = KDTree(self._array.data['position'])

        neighbours = self._kd_tree.search(point, count, radius, sort)
        return self.Neighbours(neighbours, self._array.data,
                               self._nani.element_view)

    def step(self):
        """Advance the simulation by one step.

        This is where the velocity and position for each particle are solved.
        """
        self._time += self.time_step

        if self._presolve_callback:
            self._presolve_callback(self)
            self.consolidate()

        particles = self._array.data
        _solve(particles, self.time_step)
        particles['force'] = 0

        if self._postsolve_callback:
            self._postsolve_callback(self)
            self.consolidate()

    def consolidate(self):
        """Execute the addition and removal of particles.

        Warning
        -------
        This operation is likely to invalidate any previous reference to
        particles or to query objects.

        >>> import hienoi.dynamics
        >>> sim = hienoi.dynamics.ParticleSimulation()
        >>> p0 = sim.add_particle()
        >>> sim.consolidate()
        >>> p0 = sim.get_particle(0)
        >>> p0.force = [1.0, 1.0]

        In the example above, the particle with ID 0 needs to be retrieved
        again after consolidation. Otherwise the new force set would have been
        applied to an invalid particle.
        """
        # POSTCONDITION: `self._array.data` sorted by id.
        chunks = ([self._array.data]
                  + [chunk for chunk in self._buffer.chunks])
        filters = [chunk['alive'] for chunk in chunks]
        counts = [numpy.count_nonzero(filter) for filter in filters]
        new_size = sum(counts)

        if (len(self._buffer) == 0
                and new_size == len(self._array)):
            return

        self._array.resize(new_size, copy=False)
        array = self._array.data

        i = 0
        for filter, chunk, count in _zip(filters, chunks, counts):
            if count == len(chunk):
                array[i:i + count] = chunk
            else:
                numpy.compress(filter, chunk, out=array[i:i + count])

            i += count

        self._buffer.clear()
        self._kd_tree = None


def _solve(particles, time_step):
    """Solve the particles."""
    # Implemented as a simple Euler integration.
    particles['velocity'] += (particles['force'] * time_step
                              / particles['mass'][:, numpy.newaxis])
    particles['position'] += particles['velocity'] * time_step



#!/usr/bin/env python

"""Particles orbiting around the origin.

Features:

- user attributes: particles are initialized within a radius from the
  origin and are, at each simulation step, updated to orbit around the origin.
- NumPy: operations are done directly on the particle data for increased
  performances.
"""

import math
import os
import sys

import numpy

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

import hienoi.application
from hienoi import Vector2f, Vector4f


# Mass of the central object around which particles are orbiting.
_CENTRAL_MASS = 50.0
# Number of particles.
_COUNT = 1000
# Minimum radius of the disc used to distribute the particles.
_MIN_RADIUS = 2.0
# Maximum radius of the disc used to distribute the particles.
_MAX_RADIUS = 30.0
# Mass of each particle.
_MASS = 2.0
# Mass variance for each particle.
_MASS_VARIANCE = 1.0
# Size of a particle, relative to its mass.
_SIZE = 0.2
# Squared distance to the origin where particles are drawn in the 'far color'.
_FAR_SQUARED_DISTANCE = 500.0
# Color to use for far particles.
_FAR_COLOR = Vector4f(0.0, 1.0, 1.0, 1.0)
# Color to use for near particles.
_NEAR_COLOR = Vector4f(1.0, 0.0, 0.0, 1.0)


def initialize_particle_simulation(sim):
    """Callback to initialize the particle simulation state.

    Parameters
    ----------
    sim : hienoi.dynamics.ParticleSimulation
        Particle simulation.
    """
    numpy.random.seed(_COUNT + 611)

    # Add a few particles at random positions within a given radius and with
    # initial velocities suitable for elliptical orbiting.
    particles = sim.add_particles(_COUNT)
    data = particles.data

    r = numpy.random.uniform(low=_MIN_RADIUS, high=_MAX_RADIUS, size=_COUNT)
    t = numpy.random.uniform(high=2.0 * numpy.pi, size=_COUNT)

    data['position'][:, 0] = r * numpy.cos(t)
    data['position'][:, 1] = r * numpy.sin(t)

    data['mass'] = numpy.random.uniform(low=_MASS - _MASS_VARIANCE,
                                        high=_MASS + _MASS_VARIANCE,
                                        size=_COUNT)

    speeds = numpy.sqrt(data['mass'] / r)
    data['velocity'][:, 0] = data['position'][:, 1] * speeds
    data['velocity'][:, 1] = -data['position'][:, 0] * speeds

    data['size'] = data['mass'] * _SIZE / _MASS


def update_particle_simulation(sim):
    """Callback to update the particle simulation state.

    Parameters
    ----------
    sim : hienoi.dynamics.ParticleSimulation
        Particle simulation.
    """
    data = sim.particles.data
    squared_distances = numpy.sum(data['position'][numpy.newaxis, :] ** 2,
                                  axis=-1)
    squared_distances = squared_distances.reshape(-1, 1)

    data['force'] -= (data['position']
                      * _CENTRAL_MASS
                      * data['mass'][:, numpy.newaxis]
                      / squared_distances)

    data['color'] = (_FAR_COLOR - _NEAR_COLOR)
    data['color'] *= squared_distances / _FAR_SQUARED_DISTANCE
    data['color'] += _NEAR_COLOR


def run():
    """Run the application."""
    return hienoi.application.run(
        gui = {
            'window_title': 'orbit',
            'show_grid': False,
        },
        particle_simulation={
            'initialize_callback': initialize_particle_simulation,
            'postsolve_callback': update_particle_simulation,
        })


if __name__ == '__main__':
    sys.exit(run())



"""Glue for the Nani library."""

__all__ = ['Bool', 'Object', 'Number', 'String', 'Unicode', 'Array',
           'Structure', 'Bytes', 'Str', 'Field', 'READ_ONLY']

import collections
import sys

import nani

import hienoi._numeric


_PY2 = sys.version_info[0] == 2


if _PY2:
    _BUILTIN_MODULE = '__builtin__'
    _BuiltinString = str
    _BuiltinUnicode = unicode
    _range = xrange
else:
    _BUILTIN_MODULE = 'builtins'
    _BuiltinString = bytes
    _BuiltinUnicode = str
    _range = range


assert nani.Bool._fields == ('default', 'view')
assert nani.Object._fields == ('default', 'view')
assert nani.Number._fields == ('type', 'default', 'view')
assert nani.String._fields == ('length', 'default', 'view')
assert nani.Unicode._fields == ('length', 'default', 'view')
assert nani.Array._fields == ('element_type', 'shape', 'name', 'view')
assert nani.Structure._fields == ('fields', 'name', 'view')
assert nani.Field._fields == ('name', 'type', 'read_only')


class Bool(nani.Bool):
    """Type corresponding to a boolean.

    Attributes
    ----------
    default : bool
        Default value.
    view : type or None
        If ``None``, the owning array returns a direct reference to this
        boolean value, otherwise it is expected to be a class object wrapping
        it and accepting 2 parameters: ``data``, the NumPy array owning the
        boolean value, and ``index``, its position in the array.
    """

    __slots__ = ()


Bool.__new__.__defaults__ = (False, None)


class Object(nani.Object):
    """Type corresponding to a Python object.

    Attributes
    ----------
    default : object
        Default value.
    view : type or None
        If ``None``, the owning array returns a direct reference to this Python
        object, otherwise it is expected to be a class object wrapping it and
        accepting 2 parameters: ``data``, the NumPy array owning the Python
        object, and ``index``, its position in the array.
    """

    __slots__ = ()


Object.__new__.__defaults__ = (None, None)


class Number(nani.Number):
    """Type corresponding to a number.

    Attributes
    ----------
    type : hienoi numerical type
        Type of the number. Either one from :class:`Int8`, :class:`UInt8`,
        :class:`Int16`, :class:`UInt16`, :class:`Int32`, :class:`UInt32`,
        :class:`Float32`, or :class:`Float64`.
    default : numbers.Number or numpy.number
        Default value.
    view : type or None
        If ``None``, the owning array returns a direct reference to this
        numeric value, otherwise it is expected to be a class object wrapping
        it and accepting 2 parameters: ``data``, the NumPy array owning the
        numeric value, and ``index``, its position in the array.
    """

    __slots__ = ()


Number.__new__.__defaults__ = (hienoi._numeric.Float32, 0, None)


class String(nani.String):
    """Type corresponding to a string.

    Attributes
    ----------
    length : int
        Number of characters.
    default : str on PY2 or bytes on PY3
        Default value.
    view : type or None
        If ``None``, the owning array returns a direct reference to this string
        value, otherwise it is expected to be a class object wrapping it and
        accepting 2 parameters: ``data``, the NumPy array owning the string
        value, and ``index``, its position in the array.
    """

    __slots__ = ()


String.__new__.__defaults__ = (_BuiltinString(), None)


class Unicode(nani.Unicode):
    """Type corresponding to a unicode string.

    Attributes
    ----------
    length : int
        Number of characters.
    default : unicode on PY2 or str on PY3
        Default value.
    view : type or None
        If ``None``, the owning array returns a direct reference to this
        unicode value, otherwise it is expected to be a class object wrapping
        it and accepting 2 parameters: ``data``, the NumPy array owning the
        unicode value, and ``index``, its position in the array.
    """

    __slots__ = ()


Unicode.__new__.__defaults__ = (_BuiltinUnicode(), None)


class Array(nani.Array):
    """Type corresponding to an array.

    Attributes
    ----------
    element_type : hienoi type
        Type of each element.
    shape : int or tuple of int
        Shape of the array. Passing an int defines a 1D array.
    name : str or None
        Name for the view type if `view` is ``None``.
    view : type or None
        If ``None``, a view for this array is dynamically generated by Nani,
        otherwise it is expected to be a class object wrapping it and accepting
        1 parameter: ``data``, the corresponding NumPy array.
    """

    __slots__ = ()


Array.__new__.__defaults__ = (None, None)


class Structure(nani.Structure):
    """Type corresponding to a structure.

    Attributes
    ----------
    fields : tuple of hienoi.Field or compatible tuple
        Fields defining the structure.
    name : str or None
        Name for the view type if `view` is ``None``.
    view : type or None
        If ``None``, a view for this structured array is dynamically generated
        by Nani, otherwise it is expected to be a class object wrapping it and
        accepting 1 parameter: ``data``, the corresponding NumPy structured
        array.
    """

    __slots__ = ()


Structure.__new__.__defaults__ = (None, None)


# Aliases.
Bytes = String
if _PY2:
    Str = String
else:
    Str = Unicode


_ALL_TYPES = (Bool, Object, Number, String, Unicode, Array, Structure)


class Field(nani.Field):
    """Describe a field of a structured array.

    Attributes
    ----------
    name : str
        Name of the field.
    type : nani data type
        Type of the field.
    read_only : bool
        ``True`` to not define a setter property in the structured array view
        if it is set to be dynamically generated by Nani.
    """

    __slots__ = ()


Field.__new__.__defaults__ = (False,)


READ_ONLY = nani.READ_ONLY


_FIELD_TYPE = Field._fields.index('type')
_FIELD_ATTR_COUNT = len(Field._fields)


Nani = collections.namedtuple(
    'Nani', (
        'dtype',
        'default',
        'view',
        'element_view'
    ))


class PickableNaniStructure(object):
    """Nani structure that can be pickled."""

    def __init__(self, fields, name):
        self._fields = fields
        self._name = name
        self._build()

    def __getstate__(self):
        return {'_fields': self._fields, '_name': self._name}

    def __setstate__(self, state):
        self.__dict__.update(state)
        self._build()

    def _build(self):
        self._obj = resolve(Structure(fields=self._fields, name=self._name))
        setattr(self._obj.view, 'data', property(fget=lambda self: self._data))

    @property
    def dtype(self):
        return self._obj.dtype

    @property
    def default(self):
        return self._obj.default

    @property
    def view(self):
        return self._obj.view

    @property
    def element_view(self):
        return self._obj.element_view


def resolve(data_type, name=None):
    """Retrieve the properties of a given data type."""
    _validate(data_type)
    data_type = _consolidate(data_type)
    nani.validate(data_type)
    properties = nani.resolve(data_type, name=name)
    return Nani(
        dtype=properties.dtype,
        default=properties.default,
        view=properties.view,
        element_view=nani.get_element_view(properties.view))


def _validate(data_type):
    """Check if the data type is well-formed."""
    if not isinstance(data_type, _ALL_TYPES):
        raise TypeError(
            "The data type '%s' isn't supported. It is expected to be an "
            "instance object of type %s."
            % (type(data_type).__name__, _join_types(_ALL_TYPES, "or ")))

    if isinstance(data_type, Array):
        _validate(data_type.element_type)
    elif isinstance(data_type, Structure):
        for field in data_type.fields:
            _validate(field[_FIELD_TYPE])


def _consolidate(data_type):
    """Enforce the structure of the data type.

    Specifically, convert any hienoi type into its NumPy equivalent.
    """
    def get_field_item(field, i):
        if i < len(field):
            return field[i]
        else:
            i -= _FIELD_ATTR_COUNT
            return Field.__new__.__defaults__[i]

    if isinstance(data_type, Number):
        out = data_type._replace(type=hienoi._numeric.to_numpy(data_type.type))
    elif isinstance(data_type, Array):
        out = data_type._replace(
            element_type=_consolidate(data_type.element_type))
    elif isinstance(data_type, Structure):
        fields = tuple(
            Field(*(_consolidate(get_field_item(field, i))
                    if i == _FIELD_TYPE else get_field_item(field, i)
                    for i in _range(_FIELD_ATTR_COUNT)))
            for field in data_type.fields)
        out = data_type._replace(fields=fields)
    else:
        out = data_type

    return out


def _format_type(cls):
    """Format a type name for printing."""
    if cls.__module__ == _BUILTIN_MODULE:
        return cls.__name__
    else:
        return '%s.%s' % (cls.__module__, cls.__name__)


def _format_element(element, count, idx, last_separator):
    """Format an element from a sequence.

    This only prepends a separator for the last element and wraps each element
    with single quotes.
    """
    return ("%s'%s'" % (last_separator, element)
            if count > 1 and idx == count - 1
            else "'%s'" % (element,))


def _join_sequence(seq, last_separator=''):
    """Join a sequence into a string."""
    count = len(seq)
    return ', '.join(_format_element(element, count, i, last_separator)
                     for i, element in enumerate(seq))


def _join_types(seq, last_separator=''):
    """Join class object names into a string."""
    class_names = [_format_type(cls) for cls in seq]
    return _join_sequence(class_names, last_separator)



#    __    __                   __
#   |  |--|__.-----.-----.-----|__|
#   |     |  |  -__|     |  _  |  |
#   |__|__|__|_____|__|__|_____|__|
#

"""Playground to experiment with 2D particles."""

__title__ = 'hienoi'
__version__ = '0.2.0'
__summary__ = "2D particle playground"
__url__ = 'https://github.com/christophercrouzet/hienoi'
__author__ = "Christopher Crouzet"
__contact__ = 'christopher.crouzet@gmail.com'
__license__ = "MIT"

from hienoi._common import ParticleDisplay
from hienoi._nani import *
from hienoi._numeric import *
from hienoi._vectors import *



"""Vector classes."""

__all__ = [
    'Vector2b', 'Vector2ub', 'Vector2s', 'Vector2us',
    'Vector2i', 'Vector2ui', 'Vector2f', 'Vector2d',
    'Vector3b', 'Vector3ub', 'Vector3s', 'Vector3us',
    'Vector3i', 'Vector3ui', 'Vector3f', 'Vector3d',
    'Vector4b', 'Vector4ub', 'Vector4s', 'Vector4us',
    'Vector4i', 'Vector4ui', 'Vector4f', 'Vector4d',
    'VECTOR2B', 'VECTOR2UB', 'VECTOR2S', 'VECTOR2US',
    'VECTOR2I', 'VECTOR2UI', 'VECTOR2F', 'VECTOR2D',
    'VECTOR3B', 'VECTOR3UB', 'VECTOR3S', 'VECTOR3US',
    'VECTOR3I', 'VECTOR3UI', 'VECTOR3F', 'VECTOR3D',
    'VECTOR4B', 'VECTOR4UB', 'VECTOR4S', 'VECTOR4US',
    'VECTOR4I', 'VECTOR4UI', 'VECTOR4F', 'VECTOR4D',
    'COLOR3UB', 'COLOR3F', 'COLOR4UB', 'COLOR4F',
]

import math
import sys

import hienoi._nani
import hienoi._numeric


if sys.version_info[0] == 2:
    def _iteritems(d, **kwargs):
        return d.iteritems(**kwargs)
else:
    def _iteritems(d, **kwargs):
        return iter(d.items(**kwargs))


_PATTERN = 'Vector%d'
_CTYPE_PATTERN = 'Vector%d%s'
_VIEW_PATTERN = 'Vector%d%sView'
_TOLERANCE = 1e-6


class _BaseMixin(object):
    """Vector's base mixin."""

    @classmethod
    def Vector2_lerp(cls, a, b, blend):
        return cls._class(a[0] + (b[0] - a[0]) * blend,
                          a[1] + (b[1] - a[1]) * blend)
        a + (b - a) * blend

    @classmethod
    def Vector3_lerp(cls, a, b, blend):
        return cls._class(a[0] + (b[0] - a[0]) * blend,
                          a[1] + (b[1] - a[1]) * blend,
                          a[2] + (b[2] - a[2]) * blend)

    @classmethod
    def Vector4_lerp(cls, a, b, blend):
        return cls._class(a[0] + (b[0] - a[0]) * blend,
                          a[1] + (b[1] - a[1]) * blend,
                          a[2] + (b[2] - a[2]) * blend,
                          a[3] + (b[3] - a[3]) * blend)

    def Vector2___repr__(self):
        return "%s(%s, %s)" % (self._class.__name__, self[0], self[1])

    def Vector3___repr__(self):
        return "%s(%s, %s, %s)" % (self._class.__name__,
                                   self[0], self[1], self[2])

    def Vector4___repr__(self):
        return "%s(%s, %s, %s, %s)" % (self._class.__name__,
                                       self[0], self[1], self[2], self[3])

    def Vector2___eq__(self, other):
        return self[0] == other[0] and self[1] == other[1]

    def Vector3___eq__(self, other):
        return (self[0] == other[0] and self[1] == other[1]
                and self[2] == other[2])

    def Vector4___eq__(self, other):
        return (self[0] == other[0] and self[1] == other[1]
                and self[2] == other[2] and self[3] == other[3])

    def Vector2___ne__(self, other):
        return self[0] != other[0] or self[1] != other[1]

    def Vector3___ne__(self, other):
        return (self[0] != other[0] or self[1] != other[1]
                or self[2] != other[2])

    def Vector4___ne__(self, other):
        return (self[0] != other[0] or self[1] != other[1]
                or self[2] != other[2] or self[3] != other[3])

    def Vector2___add__(self, other):
        return self._class(self[0] + other[0], self[1] + other[1])

    def Vector3___add__(self, other):
        return self._class(self[0] + other[0], self[1] + other[1],
                           self[2] + other[2])

    def Vector4___add__(self, other):
        return self._class(self[0] + other[0], self[1] + other[1],
                           self[2] + other[2], self[3] + other[3])

    def Vector2___sub__(self, other):
        return self._class(self[0] - other[0], self[1] - other[1])

    def Vector3___sub__(self, other):
        return self._class(self[0] - other[0], self[1] - other[1],
                           self[2] - other[2])

    def Vector4___sub__(self, other):
        return self._class(self[0] - other[0], self[1] - other[1],
                           self[2] - other[2], self[3] - other[3])

    def Vector2___mul__(self, other):
        return self._class(self[0] * other[0], self[1] * other[1])

    def Vector3___mul__(self, other):
        return self._class(self[0] * other[0], self[1] * other[1],
                           self[2] * other[2])

    def Vector4___mul__(self, other):
        return self._class(self[0] * other[0], self[1] * other[1],
                           self[2] * other[2], self[3] * other[3])

    def Vector2___truediv__(self, other):
        return self._class(self[0] / other[0], self[1] / other[1])

    def Vector3___truediv__(self, other):
        return self._class(self[0] / other[0], self[1] / other[1],
                           self[2] / other[2])

    def Vector4___truediv__(self, other):
        return self._class(self[0] / other[0], self[1] / other[1],
                           self[2] / other[2], self[3] / other[3])

    def Vector2___floordiv__(self, other):
        return self._class(self[0] // other[0], self[1] // other[1])

    def Vector3___floordiv__(self, other):
        return self._class(self[0] // other[0], self[1] // other[1],
                           self[2] // other[2])

    def Vector4___floordiv__(self, other):
        return self._class(self[0] // other[0], self[1] // other[1],
                           self[2] // other[2], self[3] // other[3])

    Vector2___div__ = Vector2___truediv__
    Vector3___div__ = Vector3___truediv__
    Vector4___div__ = Vector4___truediv__

    Vector2___radd__ = Vector2___add__
    Vector3___radd__ = Vector3___add__
    Vector4___radd__ = Vector4___add__

    def Vector2___rsub__(self, other):
        return self._class(other[0] - self[0], other[1] - self[1])

    def Vector3___rsub__(self, other):
        return self._class(other[0] - self[0], other[1] - self[1],
                           other[2] - self[2])

    def Vector4___rsub__(self, other):
        return self._class(other[0] - self[0], other[1] - self[1],
                           other[2] - self[2], other[3] - self[3])

    Vector2___rmul__ = Vector2___mul__
    Vector3___rmul__ = Vector3___mul__
    Vector4___rmul__ = Vector4___mul__

    def Vector2___rtruediv__(self, other):
        return self._class(other[0] / self[0], other[1] / self[1])

    def Vector3___rtruediv__(self, other):
        return self._class(other[0] / self[0], other[1] / self[1],
                           other[2] / self[2])

    def Vector4___rtruediv__(self, other):
        return self._class(other[0] / self[0], other[1] / self[1],
                           other[2] / self[2], other[3] / self[3])

    def Vector2___rfloordiv__(self, other):
        return self._class(other[0] // self[0], other[1] // self[1])

    def Vector3___rfloordiv__(self, other):
        return self._class(other[0] // self[0], other[1] // self[1],
                           other[2] // self[2])

    def Vector4___rfloordiv__(self, other):
        return self._class(other[0] // self[0], other[1] // self[1],
                           other[2] // self[2], other[3] // self[3])

    Vector2___rdiv__ = Vector2___rtruediv__
    Vector3___rdiv__ = Vector3___rtruediv__
    Vector4___rdiv__ = Vector4___rtruediv__

    def Vector2___iadd__(self, other):
        self[0] += other[0]
        self[1] += other[1]
        return self

    def Vector3___iadd__(self, other):
        self[0] += other[0]
        self[1] += other[1]
        self[2] += other[2]
        return self

    def Vector4___iadd__(self, other):
        self[0] += other[0]
        self[1] += other[1]
        self[2] += other[2]
        self[3] += other[3]
        return self

    def Vector2___isub__(self, other):
        self[0] -= other[0]
        self[1] -= other[1]
        return self

    def Vector3___isub__(self, other):
        self[0] -= other[0]
        self[1] -= other[1]
        self[2] -= other[2]
        return self

    def Vector4___isub__(self, other):
        self[0] -= other[0]
        self[1] -= other[1]
        self[2] -= other[2]
        self[3] -= other[3]
        return self

    def Vector2___imul__(self, other):
        self[0] *= other[0]
        self[1] *= other[1]
        return self

    def Vector3___imul__(self, other):
        self[0] *= other[0]
        self[1] *= other[1]
        self[2] *= other[2]
        return self

    def Vector4___imul__(self, other):
        self[0] *= other[0]
        self[1] *= other[1]
        self[2] *= other[2]
        self[3] *= other[3]
        return self

    def Vector2___itruediv__(self, other):
        self[0] /= other[0]
        self[1] /= other[1]
        return self

    def Vector3___itruediv__(self, other):
        self[0] /= other[0]
        self[1] /= other[1]
        self[2] /= other[2]
        return self

    def Vector4___itruediv__(self, other):
        self[0] /= other[0]
        self[1] /= other[1]
        self[2] /= other[2]
        self[3] /= other[3]
        return self

    def Vector2___ifloordiv__(self, other):
        self[0] //= other[0]
        self[1] //= other[1]
        return self

    def Vector3___ifloordiv__(self, other):
        self[0] //= other[0]
        self[1] //= other[1]
        self[2] //= other[2]
        return self

    def Vector4___ifloordiv__(self, other):
        self[0] //= other[0]
        self[1] //= other[1]
        self[2] //= other[2]
        self[3] //= other[3]
        return self

    Vector2___idiv__ = Vector2___itruediv__
    Vector3___idiv__ = Vector3___itruediv__
    Vector4___idiv__ = Vector4___itruediv__

    def Vector2___neg__(self):
        return self._class(-self[0], -self[1])

    def Vector3___neg__(self):
        return self._class(-self[0], -self[1], -self[2])

    def Vector4___neg__(self):
        return self._class(-self[0], -self[1], -self[2], -self[3])

    def Vector2___pos__(self):
        return self._class(+self[0], +self[1])

    def Vector3___pos__(self):
        return self._class(+self[0], +self[1], +self[2])

    def Vector4___pos__(self):
        return self._class(+self[0], +self[1], +self[2], +self[3])

    def Vector2___abs__(self):
        return self._class(abs(self[0]), abs(self[1]))

    def Vector3___abs__(self):
        return self._class(abs(self[0]), abs(self[1]), abs(self[2]))

    def Vector4___abs__(self):
        return self._class(abs(self[0]), abs(self[1]), abs(self[2]),
                           abs(self[3]))

    def Vector2_offset(self, value):
        return self._class(self[0] + value, self[1] + value)

    def Vector3_offset(self, value):
        return self._class(self[0] + value, self[1] + value, self[2] + value)

    def Vector4_offset(self, value):
        return self._class(self[0] + value, self[1] + value, self[2] + value,
                           self[3] + value)

    def Vector2_scale(self, value):
        return self._class(self[0] * value, self[1] * value)

    def Vector3_scale(self, value):
        return self._class(self[0] * value, self[1] * value, self[2] * value)

    def Vector4_scale(self, value):
        return self._class(self[0] * value, self[1] * value, self[2] * value,
                           self[3] * value)

    def Vector2_ioffset(self, value):
        self[0] += value
        self[1] += value
        return self

    def Vector3_ioffset(self, value):
        self[0] += value
        self[1] += value
        self[2] += value
        return self

    def Vector4_ioffset(self, value):
        self[0] += value
        self[1] += value
        self[2] += value
        self[3] += value
        return self

    def Vector2_iscale(self, value):
        self[0] *= value
        self[1] *= value
        return self

    def Vector3_iscale(self, value):
        self[0] *= value
        self[1] *= value
        self[2] *= value
        return self

    def Vector4_iscale(self, value):
        self[0] *= value
        self[1] *= value
        self[2] *= value
        self[3] *= value
        return self

    def Vector2_length(self):
        return math.sqrt(self[0] ** 2 + self[1] ** 2)

    def Vector3_length(self):
        return math.sqrt(self[0] ** 2 + self[1] ** 2 + self[2] ** 2)

    def Vector4_length(self):
        return math.sqrt(self[0] ** 2 + self[1] ** 2 + self[2] ** 2
                         + self[3] ** 2)

    def Vector2_squared_length(self):
        return self[0] ** 2 + self[1] ** 2

    def Vector3_squared_length(self):
        return self[0] ** 2 + self[1] ** 2 + self[2] ** 2

    def Vector4_squared_length(self):
        return self[0] ** 2 + self[1] ** 2 + self[2] ** 2 + self[3] ** 2

    def Vector2_dot(self, other):
        return self[0] * other[0] + self[1] * other[1]

    def Vector3_dot(self, other):
        return self[0] * other[0] + self[1] * other[1] + self[2] * other[2]

    def Vector4_dot(self, other):
        return (self[0] * other[0] + self[1] * other[1] + self[2] * other[2]
                + self[3] * other[3])


class _FloatingPointMixin(object):
    """Vector's floating-point mixin."""

    def Vector2_is_almost_equal(self, other, tolerance=_TOLERANCE):
        return (abs(self[0] - other[0]) <= tolerance
                and abs(self[1] - other[1]) <= tolerance)

    def Vector3_is_almost_equal(self, other, tolerance=_TOLERANCE):
        return (abs(self[0] - other[0]) <= tolerance
                and abs(self[1] - other[1]) <= tolerance
                and abs(self[2] - other[2]) <= tolerance)

    def Vector4_is_almost_equal(self, other, tolerance=_TOLERANCE):
        return (abs(self[0] - other[0]) <= tolerance
                and abs(self[1] - other[1]) <= tolerance
                and abs(self[2] - other[2]) <= tolerance
                and abs(self[3] - other[3]) <= tolerance)

    def Vector2_normalize(self):
        length = self.length()
        if abs(length) <= _TOLERANCE:
            return self._class(self[0], self[1])

        return self._class(self[0] / length, self[1] / length)

    def Vector3_normalize(self):
        length = self.length()
        if abs(length) <= _TOLERANCE:
            return self._class(self[0], self[1], self[2])

        return self._class(self[0] / length, self[1] / length,
                           self[2] / length)

    def Vector4_normalize(self):
        length = self.length()
        if abs(length) <= _TOLERANCE:
            return self._class(self[0], self[1], self[2], self[3])

        return self._class(self[0] / length, self[1] / length,
                           self[2] / length, self[3] / length)

    def Vector2_inormalize(self):
        length = self.length()
        if abs(length) <= _TOLERANCE:
            return self

        self[0] /= length
        self[1] /= length
        return self

    def Vector3_inormalize(self):
        length = self.length()
        if abs(length) <= _TOLERANCE:
            return self

        self[0] /= length
        self[1] /= length
        self[2] /= length
        return self

    def Vector4_inormalize(self):
        length = self.length()
        if abs(length) <= _TOLERANCE:
            return self

        self[0] /= length
        self[1] /= length
        self[2] /= length
        self[3] /= length
        return self


class _CtypeMixin(object):
    """Vector's ctype mixin."""

    @property
    def Vector2_x(self):
        return self[0]

    @property
    def Vector3_x(self):
        return self[0]

    @property
    def Vector4_x(self):
        return self[0]

    @Vector2_x.setter
    def Vector2_x(self, value):
        self[0] = value

    @Vector3_x.setter
    def Vector3_x(self, value):
        self[0] = value

    @Vector4_x.setter
    def Vector4_x(self, value):
        self[0] = value

    @property
    def Vector2_y(self):
        return self[1]

    @property
    def Vector3_y(self):
        return self[1]

    @property
    def Vector4_y(self):
        return self[1]

    @Vector2_y.setter
    def Vector2_y(self, value):
        self[1] = value

    @Vector3_y.setter
    def Vector3_y(self, value):
        self[1] = value

    @Vector4_y.setter
    def Vector4_y(self, value):
        self[1] = value

    @property
    def Vector3_z(self):
        return self[2]

    @property
    def Vector4_z(self):
        return self[2]

    @Vector3_z.setter
    def Vector3_z(self, value):
        self[2] = value

    @Vector4_z.setter
    def Vector4_z(self, value):
        self[2] = value

    @property
    def Vector4_w(self):
        return self[3]

    @Vector4_w.setter
    def Vector4_w(self, value):
        self[3] = value

    Vector3_r = Vector3_x
    Vector4_r = Vector4_x

    Vector3_g = Vector3_y
    Vector4_g = Vector4_y

    Vector3_b = Vector3_z
    Vector4_b = Vector4_z

    Vector4_a = Vector4_w

    def Vector2_assign(self, other):
        self[0] = other[0]
        self[1] = other[1]

    def Vector3_assign(self, other):
        self[0] = other[0]
        self[1] = other[1]
        self[2] = other[2]

    def Vector4_assign(self, other):
        self[0] = other[0]
        self[1] = other[1]
        self[2] = other[2]
        self[3] = other[3]

    def Vector2_set(self, x, y):
        self[:] = (x, y)

    def Vector3_set(self, x, y, z):
        self[:] = (x, y, z)

    def Vector4_set(self, x, y, z, w):
        self[:] = (x, y, z, w)


class _ViewMixin(object):
    """Vector's view mixin."""

    Vector2___slots__ = ('_data',)
    Vector3___slots__ = ('_data',)
    Vector4___slots__ = ('_data',)

    def Vector2___init__(self, data):
        self._data = data

    def Vector3___init__(self, data):
        self._data = data

    def Vector4___init__(self, data):
        self._data = data

    def Vector2___len__(self):
        return 2

    def Vector3___len__(self):
        return 3

    def Vector4___len__(self):
        return 4

    def Vector2___getitem__(self, idx):
        return self._data[idx]

    def Vector3___getitem__(self, idx):
        return self._data[idx]

    def Vector4___getitem__(self, idx):
        return self._data[idx]

    def Vector2___setitem__(self, idx, value):
        self._data[idx] = value

    def Vector3___setitem__(self, idx, value):
        self._data[idx] = value

    def Vector4___setitem__(self, idx, value):
        self._data[idx] = value

    @property
    def Vector2_x(self):
        return self._data[0]

    @property
    def Vector3_x(self):
        return self._data[0]

    @property
    def Vector4_x(self):
        return self._data[0]

    @Vector2_x.setter
    def Vector2_x(self, value):
        self._data[0] = value

    @Vector3_x.setter
    def Vector3_x(self, value):
        self._data[0] = value

    @Vector4_x.setter
    def Vector4_x(self, value):
        self._data[0] = value

    @property
    def Vector2_y(self):
        return self._data[1]

    @property
    def Vector3_y(self):
        return self._data[1]

    @property
    def Vector4_y(self):
        return self._data[1]

    @Vector2_y.setter
    def Vector2_y(self, value):
        self._data[1] = value

    @Vector3_y.setter
    def Vector3_y(self, value):
        self._data[1] = value

    @Vector4_y.setter
    def Vector4_y(self, value):
        self._data[1] = value

    @property
    def Vector3_z(self):
        return self._data[2]

    @property
    def Vector4_z(self):
        return self._data[2]

    @Vector3_z.setter
    def Vector3_z(self, value):
        self._data[2] = value

    @Vector4_z.setter
    def Vector4_z(self, value):
        self._data[2] = value

    @property
    def Vector4_w(self):
        return self._data[3]

    @Vector4_w.setter
    def Vector4_w(self, value):
        self._data[3] = value

    Vector3_r = Vector3_x
    Vector4_r = Vector4_x

    Vector3_g = Vector3_y
    Vector4_g = Vector4_y

    Vector3_b = Vector3_z
    Vector4_b = Vector4_z

    Vector4_a = Vector4_w

    def Vector2_assign(self, other):
        self._data[:] = other._data

    def Vector3_assign(self, other):
        self._data[:] = other._data

    def Vector4_assign(self, other):
        self._data[:] = other._data

    def Vector2_set(self, x, y):
        self._data[:] = (x, y)

    def Vector3_set(self, x, y, z):
        self._data[:] = (x, y, z)

    def Vector4_set(self, x, y, z, w):
        self._data[:] = (x, y, z, w)


_VECTOR_MIXINS = {
    hienoi._numeric.Int8: (_BaseMixin,),
    hienoi._numeric.UInt8: (_BaseMixin,),
    hienoi._numeric.Int16: (_BaseMixin,),
    hienoi._numeric.UInt16: (_BaseMixin,),
    hienoi._numeric.Int32: (_BaseMixin,),
    hienoi._numeric.UInt32: (_BaseMixin,),
    hienoi._numeric.Float32: (_BaseMixin, _FloatingPointMixin),
    hienoi._numeric.Float64: (_BaseMixin, _FloatingPointMixin),
}


def _define_vectors(size):
    """Define a triplet of objects describing a vector."""
    out = {}
    for element_type in hienoi._numeric.get_types():
        mixins = _VECTOR_MIXINS[type] = _VECTOR_MIXINS[element_type]
        literal = hienoi._numeric.get_type_literal(element_type)
        ctype_name = _CTYPE_PATTERN % (size, literal)
        ctype = _define_ctype(ctype_name, element_type, mixins, size)
        view_name = _VIEW_PATTERN % (size, literal)
        view = _define_view(view_name, ctype, mixins, size)
        nani = _define_nani(element_type, view, size)
        out[element_type] = (ctype, view, nani)

    return out


def _define_ctype(name, element_type, mixins, size):
    """Define a vector's ctype."""
    element_type = hienoi._numeric.to_ctype(element_type)
    mixins = (_CtypeMixin,) + mixins
    definition = _get_definition(mixins, size)

    # Required for Python 2.
    definition.update({
        '_length_': size,
        '_type_': element_type,
    })

    out = type(name, (element_type * size,), definition)
    out._class = out
    return out


def _define_view(name, ctype, mixins, size):
    """Define a vector's view."""
    mixins = (_ViewMixin,) + mixins
    definition = _get_definition(mixins, size)
    out = type(name, (), definition)
    out._class = ctype
    return out


def _define_nani(element_type, view, size):
    """Define a vector's nani."""
    return hienoi._nani.Array(
        element_type=hienoi._nani.Number(type=element_type),
        shape=size,
        view=view)


def _get_definition(mixins, size):
    """Retrieve a vector's definition."""
    out = {}
    prefix = _PATTERN % (size,)
    prefix_length = len(prefix)
    for mixin in mixins:
        out.update({key[prefix_length + 1:]: value
                    for key, value in _iteritems(mixin.__dict__)
                    if key.startswith(prefix)})

    return out


_VECTORS2 = _define_vectors(2)
Vector2b, Vector2bView, VECTOR2B = _VECTORS2[hienoi._numeric.Int8]
Vector2ub, Vector2ubView, VECTOR2UB = _VECTORS2[hienoi._numeric.UInt8]
Vector2s, Vector2sView, VECTOR2S = _VECTORS2[hienoi._numeric.Int16]
Vector2us, Vector2usView, VECTOR2US = _VECTORS2[hienoi._numeric.UInt16]
Vector2i, Vector2iView, VECTOR2I = _VECTORS2[hienoi._numeric.Int32]
Vector2ui, Vector2uiView, VECTOR2UI = _VECTORS2[hienoi._numeric.UInt32]
Vector2f, Vector2fView, VECTOR2F = _VECTORS2[hienoi._numeric.Float32]
Vector2d, Vector2dView, VECTOR2D = _VECTORS2[hienoi._numeric.Float64]

_VECTORS3 = _define_vectors(3)
Vector3b, Vector3bView, VECTOR3B = _VECTORS3[hienoi._numeric.Int8]
Vector3ub, Vector3ubView, VECTOR3UB = _VECTORS3[hienoi._numeric.UInt8]
Vector3s, Vector3sView, VECTOR3S = _VECTORS3[hienoi._numeric.Int16]
Vector3us, Vector3usView, VECTOR3US = _VECTORS3[hienoi._numeric.UInt16]
Vector3i, Vector3iView, VECTOR3I = _VECTORS3[hienoi._numeric.Int32]
Vector3ui, Vector3uiView, VECTOR3UI = _VECTORS3[hienoi._numeric.UInt32]
Vector3f, Vector3fView, VECTOR3F = _VECTORS3[hienoi._numeric.Float32]
Vector3d, Vector3dView, VECTOR3D = _VECTORS3[hienoi._numeric.Float64]

_VECTORS4 = _define_vectors(4)
Vector4b, Vector4bView, VECTOR4B = _VECTORS4[hienoi._numeric.Int8]
Vector4ub, Vector4ubView, VECTOR4UB = _VECTORS4[hienoi._numeric.UInt8]
Vector4s, Vector4sView, VECTOR4S = _VECTORS4[hienoi._numeric.Int16]
Vector4us, Vector4usView, VECTOR4US = _VECTORS4[hienoi._numeric.UInt16]
Vector4i, Vector4iView, VECTOR4I = _VECTORS4[hienoi._numeric.Int32]
Vector4ui, Vector4uiView, VECTOR4UI = _VECTORS4[hienoi._numeric.UInt32]
Vector4f, Vector4fView, VECTOR4F = _VECTORS4[hienoi._numeric.Float32]
Vector4d, Vector4dView, VECTOR4D = _VECTORS4[hienoi._numeric.Float64]

# Color aliases.
COLOR3UB = VECTOR3UB._replace(
    element_type=VECTOR3UB.element_type._replace(default=255))
COLOR3F = VECTOR3F._replace(
    element_type=VECTOR3F.element_type._replace(default=1.0))
COLOR4UB = VECTOR4UB._replace(
    element_type=VECTOR4UB.element_type._replace(default=255))
COLOR4F = VECTOR4F._replace(
    element_type=VECTOR4F.element_type._replace(default=1.0))



"""Glue for numeric data types from different modules."""

__all__ = ['Int8', 'UInt8', 'Int16', 'UInt16', 'Int32', 'UInt32', 'Float32',
           'Float64']

import ctypes

import numpy
import OpenGL.GL as gl


class Int8(object):
    """8-bit int type."""

    pass


class UInt8(object):
    """8-bit unsigned int type."""

    pass


class Int16(object):
    """16-bit int type."""

    pass


class UInt16(object):
    """16-bit unsigned int type."""

    pass


class Int32(object):
    """32-bit int type."""

    pass


class UInt32(object):
    """32-bit unsigned int type."""

    pass


class Float32(object):
    """32-bit floating-point type."""

    pass


class Float64(object):
    """64-bit floating-point type."""

    pass


_TYPES_DATA = {
    Int8: {
        'size': 1,
        'literal': 'b',
        'ctype': ctypes.c_int8,
        'gl': gl.GL_BYTE,
        'numpy': numpy.int8,
    },
    UInt8: {
        'size': 1,
        'literal': 'ub',
        'ctype': ctypes.c_uint8,
        'gl': gl.GL_UNSIGNED_BYTE,
        'numpy': numpy.uint8,
    },
    Int16: {
        'size': 2,
        'literal': 's',
        'ctype': ctypes.c_int16,
        'gl': gl.GL_SHORT,
        'numpy': numpy.int16,
    },
    UInt16: {
        'size': 2,
        'literal': 'us',
        'ctype': ctypes.c_uint16,
        'gl': gl.GL_UNSIGNED_SHORT,
        'numpy': numpy.uint16,
    },
    Int32: {
        'size': 4,
        'literal': 'i',
        'ctype': ctypes.c_int32,
        'gl': gl.GL_INT,
        'numpy': numpy.int32,
    },
    UInt32: {
        'size': 4,
        'literal': 'ui',
        'ctype': ctypes.c_uint32,
        'gl': gl.GL_UNSIGNED_INT,
        'numpy': numpy.uint32,
    },
    Float32: {
        'size': 4,
        'literal': 'f',
        'ctype': ctypes.c_float,
        'gl': gl.GL_FLOAT,
        'numpy': numpy.float32,
    },
    Float64: {
        'size': 8,
        'literal': 'd',
        'ctype': ctypes.c_double,
        'gl': gl.GL_DOUBLE,
        'numpy': numpy.float64,
    },
}

_TYPES = tuple(_TYPES_DATA.keys())

_CTYPE_TO_THIS = {_TYPES_DATA[this_type]['ctype']: this_type
                  for this_type in _TYPES}

_GL_TO_THIS = {_TYPES_DATA[this_type]['gl']: this_type
               for this_type in _TYPES}

_NUMPY_TO_THIS = {_TYPES_DATA[this_type]['numpy']: this_type
                  for this_type in _TYPES}


def get_types():
    """Retrieve all the numeric types supported by hienoi."""
    return _TYPES


def get_type_size(this_type):
    """Retrieve the size in bytes of a hienoi type."""
    return _TYPES_DATA[this_type]['size']


def get_type_literal(this_type):
    """Retrieve the string literal of a hienoi type."""
    return _TYPES_DATA[this_type]['literal']


def from_ctype(ctype_type):
    """Convert a C type to its hienoi equivalent."""
    return _CTYPE_TO_THIS[ctype_type]


def from_gl(gl_type):
    """Convert an OpenGL type to its hienoi equivalent."""
    return _GL_TO_THIS[gl_type]


def from_numpy(numpy_type):
    """Convert a NumPy type to its hienoi equivalent."""
    return _NUMPY_TO_THIS[numpy_type]


def to_ctype(this_type):
    """Convert a hienoi type to its ctypes equivalent."""
    return _TYPES_DATA[this_type]['ctype']


def to_gl(this_type):
    """Convert a hienoi type to its OpenGL equivalent."""
    return _TYPES_DATA[this_type]['gl']


def to_numpy(this_type):
    """Convert a hienoi type to its NumPy equivalent."""
    return _TYPES_DATA[this_type]['numpy']



#!/usr/bin/env python

import os
import sys
import unittest

import numpy

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

from hienoi._orderedbuffer import OrderedBuffer


class OrderedBufferTest(unittest.TestCase):

    def test_constructor(self):
        b = OrderedBuffer(256, numpy.dtype(numpy.float32))
        self.assertEqual(len(b), 0)
        self.assertEqual(b.bucket_capacity, 256)
        self.assertEqual(b.chunks, [])
        self.assertEqual([chunk for chunk in b.chunks], [])

    def test_append(self):
        b = OrderedBuffer(3, numpy.dtype(numpy.int8))

        data = b.append(9)
        self.assertEqual(data, 9)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(b), 1)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9]])

        data = b.append(1)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(b), 2)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1]])

        data = b.append(4)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(b), 3)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4]])

        data = b.append(0)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(b), 4)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0]])

    def test_extend(self):
        b = OrderedBuffer(3, numpy.dtype(numpy.int8))

        # Add an empty bunch.
        bunch = b.extend([])
        self.assertEqual(bunch.tolist(), [])
        self.assertEqual(len(bunch), 0)
        self.assertIsNone(bunch.base)
        self.assertEqual(len(b), 0)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [])

        # Fill the first bucket.
        bunch = b.extend([9, 1, 4])
        self.assertEqual(bunch.tolist(), [9, 1, 4])
        self.assertEqual(len(bunch), 3)
        self.assertEqual(len(bunch.base), 3)
        self.assertEqual(len(b), 3)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4]])

        # Partly fill the second bucket.
        bunch = b.extend([0, 7])
        self.assertEqual(bunch.tolist(), [0, 7])
        self.assertEqual(len(bunch), 2)
        self.assertEqual(len(bunch.base), 3)
        self.assertEqual(len(b), 5)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7]])

        # This doesn't fit without overflowing the second bucket so it is
        # stored as a bunch.
        bunch = b.extend([5, 2])
        self.assertEqual(len(bunch), 2)
        self.assertIsNone(bunch.base)
        self.assertEqual(len(b), 7)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7], [5, 2]])

        # Finish filling the second bucket while preserving the order.
        bunch = b.extend([3])
        self.assertEqual(len(bunch), 1)
        self.assertEqual(len(bunch.base), 3)
        self.assertEqual(len(b), 8)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7], [5, 2], [3]])

        # Partly fill a third bucket.
        bunch = b.extend([6])
        self.assertEqual(len(bunch), 1)
        self.assertEqual(len(bunch.base), 3)
        self.assertEqual(len(b), 9)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7], [5, 2], [3], [6]])

        # Add a bunch larger than the bucket capacity.
        bunch = b.extend([1, 2, 3, 4, 5])
        self.assertEqual(len(bunch), 5)
        self.assertIsNone(bunch.base)
        self.assertEqual(len(b), 14)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7], [5, 2], [3], [6], [1, 2, 3, 4, 5]])

        # Finish filling the third bucket.
        bunch = b.extend([7, 8])
        self.assertEqual(len(bunch), 2)
        self.assertEqual(len(bunch.base), 3)
        self.assertEqual(len(b), 16)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7], [5, 2], [3], [6], [1, 2, 3, 4, 5], [7, 8]])

        # Partly fill a fourth bucket.
        bunch = b.extend([9])
        self.assertEqual(len(bunch), 1)
        self.assertEqual(len(bunch.base), 3)
        self.assertEqual(len(b), 17)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7], [5, 2], [3], [6], [1, 2, 3, 4, 5], [7, 8], [9]])

        # Finish filling the fourth bucket.
        bunch = b.extend([8, 7])
        self.assertEqual(len(bunch), 2)
        self.assertEqual(len(bunch.base), 3)
        self.assertEqual(len(b), 19)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1, 4], [0, 7], [5, 2], [3], [6], [1, 2, 3, 4, 5], [7, 8], [9, 8, 7]])

    def test_clear(self):
        b = OrderedBuffer(3, numpy.dtype(numpy.int8))
        b.extend([9, 1])
        b.extend([1, 2, 3, 4, 5])
        b.append(4)
        self.assertEqual(len(b), 8)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[9, 1], [1, 2, 3, 4, 5], [4]])

        b.clear()
        self.assertEqual(len(b), 0)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [])

        b.append(0)
        b.extend([7, 8, 9])
        b.append(1)
        b.extend([2])
        self.assertEqual(len(b), 6)
        self.assertEqual([chunk.tolist() for chunk in b.chunks], [[0], [7, 8, 9], [1, 2]])


if __name__ == '__main__':
    from tests.run import run
    run('__main__')



"""Bucket buffer based on NumPy."""

import collections
import itertools
import sys

import numpy


if sys.version_info[0] == 2:
    _range = xrange
    _zip = itertools.izip
else:
    _range = range
    _zip = zip


_Bunch = collections.namedtuple(
    '_Bunch', (
        'position',
        'data',
    ))


class OrderedBuffer(object):
    """Ordered buffer based on NumPy.

    Prevents reallocation and hence invalidation of previously added elements.
    """

    def __init__(self, bucket_capacity, dtype):
        if bucket_capacity < 1:
            raise ValueError("A minimum bucket capacity of 1 is expected.")

        self._dtype = dtype

        self._buckets = []
        self._bucket_capacity = bucket_capacity
        self._pos = [0, 0]

        self._bunchs = []

    def __len__(self):
        return (self._bucket_capacity * self._pos[0] + self._pos[1]
                + sum(len(bunch.data) for bunch in self._bunchs))

    @property
    def chunks(self):
        out = []
        i, j = (0, 0)
        bunchs = self._bunchs + [_Bunch(position=self._pos, data=[])]
        for bunch in bunchs:
            m, n = bunch.position
            while i < m:
                out.append(self._buckets[i][j:])
                i += 1
                j = 0

            if j < n:
                out.append(self._buckets[i][j:n])

            if len(bunch.data):
                out.append(bunch.data)

            i, j = bunch.position

        return out

    @property
    def bucket_capacity(self):
        return self._bucket_capacity

    @property
    def dtype(self):
        return self._dtype

    def append(self, data):
        """Append a new element."""
        return self._push(data, 1)[0]

    def extend(self, data):
        """Append a bunch of new elements."""
        size = len(data)
        if size < 1:
            return numpy.empty(0, dtype=self._dtype)

        return self._push(numpy.asarray(data, dtype=self._dtype), size)

    def clear(self):
        """Clear the data."""
        self._pos = [0, 0]
        self._bunchs = []

    def _push(self, data, size):
        """Add new element(s) at the end."""
        i, j = self._pos
        if self._bucket_capacity - j >= size:
            if i == len(self._buckets):
                self._buckets.append(
                    numpy.empty(self._bucket_capacity, dtype=self._dtype))

            self._buckets[i][j:j + size] = data
            out = self._buckets[i][j:j + size]

            j = (j + size) % self._bucket_capacity
            self._pos = [i + 1 if j == 0 else i, j]
        else:
            self._bunchs.append(_Bunch(position=self._pos, data=data))
            out = data

        return out



"""Dynamic array based on NumPy."""

import math

import numpy


_GROW_FACTOR = 1.5
assert _GROW_FACTOR > 1.0


class DynamicArray(object):
    """Dynamic array based on NumPy.

    This represents a dynamic contiguous array along the lines of the C++
    ``std::vector`` container.
    """

    @classmethod
    def from_buffer(cls, buf, dtype):
        """Unpack a buffer object into a new array."""
        this = DynamicArray(0, dtype)
        this._array = numpy.frombuffer(buf, dtype)
        this._size = len(this._array)
        return this

    def __init__(self, capacity, dtype):
        self._array = numpy.empty(capacity, dtype=dtype)
        self._size = 0

    def __len__(self):
        return self._size

    @property
    def data(self):
        return self._array[:self._size]

    @property
    def capacity(self):
        return len(self._array)

    @property
    def dtype(self):
        return self._array.dtype

    def copy_from(self, data):
        """Replace the data with a copy of some other data."""
        size = len(data)
        self.grow(size, copy=False)
        for field in self._array.dtype.fields:
            self._array[field][:size] = data[field]

        self._size = size

    def grow(self, requested, copy=True):
        """Increase the storage's capacity if needed."""
        if requested <= len(self._array):
            return

        capacity = max(requested,
                       int(max(len(self._array) * _GROW_FACTOR,
                               math.ceil(1.0 / (_GROW_FACTOR - 1.0)) * 2)))
        array = numpy.empty(capacity, dtype=self._array.dtype)
        if copy:
            array[:self._size] = self._array[:self._size]
        else:
            self._size = 0

        self._array = array

    def resize(self, requested, copy=True):
        """Change the size of the array."""
        self.grow(requested, copy=copy)
        self._size = requested

    def append(self, data):
        """Append a new element."""
        new_size = self._size + 1
        self.grow(new_size)
        self._array[self._size] = data
        out = self._array[self._size]
        self._size = new_size
        return out

    def extend(self, data):
        """Append multiple elements."""
        new_size = self._size + len(data)
        self.grow(new_size)
        self._array[self._size:new_size] = data
        out = self._array[self._size:new_size]
        self._size = new_size
        return out

    def clear(self):
        """Clear the data."""
        self._size = 0



#!/usr/bin/env python

import os
import sys
import timeit
import unittest

import numpy

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

from hienoi._kdtree import KDTree


_clock = timeit.default_timer


def _generate_uniform_2D_data(count, seed):
    numpy.random.seed(seed=seed)
    data = numpy.empty(count, dtype=(numpy.float32, 2))
    data[:, 0] = numpy.random.uniform(-1.0, 1.0, count)
    data[:, 1] = numpy.random.uniform(-1.0, 1.0, count)
    return data


def _generate_2D_point(seed):
    numpy.random.seed(seed=seed)
    return (numpy.float32(numpy.random.random() * 2.0 - 1.0),
            numpy.float32(numpy.random.random() * 2.0 - 1.0))


class KDTreeBench(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls._trees = {
            1: KDTree(_generate_uniform_2D_data(1, 391)),
            10: KDTree(_generate_uniform_2D_data(10, 813)),
            100: KDTree(_generate_uniform_2D_data(100, 571)),
            1000: KDTree(_generate_uniform_2D_data(1000, 600)),
            10000: KDTree(_generate_uniform_2D_data(10000, 427)),
            100000: KDTree(_generate_uniform_2D_data(100000, 180)),
            1000000: KDTree(_generate_uniform_2D_data(1000000, 425)),
        }

    def bench_build_uniform_100(self):
        data = _generate_uniform_2D_data(100, 968)
        start = _clock()
        tree = KDTree(data)
        return _clock() - start

    def bench_build_uniform_10000(self):
        data = _generate_uniform_2D_data(10000, 441)
        start = _clock()
        tree = KDTree(data)
        return _clock() - start

    def bench_build_uniform_1000000(self):
        data = _generate_uniform_2D_data(1000000, 96)
        start = _clock()
        tree = KDTree(data)
        return _clock() - start

    def bench_search_uniform_1_nearests_in_100_points_tree_100_times(self):
        numpy.random.seed(seed=192)
        k = 1
        r = None
        n = 100
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_100_nearests_in_100_points_tree_100_times(self):
        numpy.random.seed(seed=501)
        k = 100
        r = None
        n = 100
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_1_nearests_in_10000_points_tree_100_times(self):
        numpy.random.seed(seed=495)
        k = 1
        r = None
        n = 10000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_100_nearests_in_10000_points_tree_100_times(self):
        numpy.random.seed(seed=201)
        k = 100
        r = None
        n = 10000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_1_nearests_in_1000000_points_tree_100_times(self):
        numpy.random.seed(seed=597)
        k = 1
        r = None
        n = 1000000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_100_nearests_in_1000000_points_tree_100_times(self):
        numpy.random.seed(seed=109)
        k = 100
        r = None
        n = 1000000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_all_within_0_25_radius_in_100_points_tree_100_times(self):
        numpy.random.seed(seed=220)
        k = None
        r = 0.25
        n = 100
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_all_within_0_75_radius_in_100_points_tree_100_times(self):
        numpy.random.seed(seed=384)
        k = None
        r = 0.75
        n = 100
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_all_within_0_25_radius_in_10000_points_tree_100_times(self):
        numpy.random.seed(seed=471)
        k = None
        r = 0.25
        n = 10000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_all_within_0_75_radius_in_10000_points_tree_100_times(self):
        numpy.random.seed(seed=173)
        k = None
        r = 0.75
        n = 10000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_all_within_0_25_radius_in_1000000_points_tree_100_times(self):
        numpy.random.seed(seed=28)
        k = None
        r = 0.25
        n = 1000000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start

    def bench_search_uniform_all_within_0_75_radius_in_1000000_points_tree_100_times(self):
        numpy.random.seed(seed=850)
        k = None
        r = 0.75
        n = 1000000
        count = 100
        tree = self._trees[n]
        points = [_generate_2D_point(seed) for seed in numpy.random.randint(999, size=count)]
        start = _clock()
        for point in points:
            tree.search(point, count=k, radius=r, sort=True)

        return _clock() - start


if __name__ == '__main__':
    from benchmarks.run import run
    run('__main__')



#!/usr/bin/env python

import os
import sys
import unittest

import numpy

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

from hienoi._dynamicarray import DynamicArray


class DynamicArrayTest(unittest.TestCase):

    def test_constructor(self):
        a = DynamicArray(0, numpy.dtype(numpy.float32))
        self.assertEqual(len(a), 0)
        self.assertEqual(a.capacity, 0)
        self.assertEqual(len(a.data), 0)
        self.assertEqual(len(a.data.base), 0)
        self.assertEqual(a.data.tolist(), [])

        a = DynamicArray(256, numpy.dtype(numpy.float32))
        self.assertEqual(len(a), 0)
        self.assertEqual(a.capacity, 256)
        self.assertEqual(len(a.data), 0)
        self.assertEqual(len(a.data.base), 256)
        self.assertEqual(a.data.tolist(), [])

    def test_data(self):
        a = DynamicArray(256, numpy.dtype(numpy.int8))
        a.extend([0, 1, 4, 9])
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])
        self.assertEqual(len(a.data.base), 256)

    def test_capacity(self):
        a = DynamicArray(0, numpy.dtype(numpy.float32))
        self.assertEqual(a.capacity, 0)
        a.extend([0, 1, 4, 9])
        self.assertGreaterEqual(a.capacity, 4)

        a = DynamicArray(256, numpy.dtype(numpy.float32))
        self.assertEqual(a.capacity, 256)
        a.extend([1, 0, 1, 4, 9])
        self.assertEqual(a.capacity, 256)

    def test_copy_from(self):
        a = DynamicArray(0, numpy.dtype([
            ('a', numpy.int8),
            ('b', numpy.float32),
            ('c', numpy.float64),
        ]))
        a.extend([
            (4, 3.932, 902.345),
            (7, 1.016, 548.229),
            (2, 0.542, 771.031),
            (8, 5.429, 858.063),
        ])
        b = DynamicArray(0, numpy.dtype([
            ('a', numpy.int8),
            ('c', numpy.float64),
        ]))
        b.copy_from(a.data)
        self.assertEqual(len(b), 4)
        self.assertEqual(b.data.tolist(), [(4, 902.345), (7, 548.229), (2, 771.031), (8, 858.063),])

    def test_grow(self):
        a = DynamicArray(0, numpy.dtype(numpy.int8))
        self.assertEqual(len(a), 0)
        self.assertEqual(len(a.data), 0)

        a.grow(4)
        self.assertEqual(len(a), 0)
        self.assertEqual(len(a.data), 0)
        self.assertGreaterEqual(a.capacity, 4)

        a.extend([0, 1, 4, 9])
        self.assertEqual(len(a), 4)
        self.assertEqual(len(a.data), 4)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])

        a.grow(2)
        self.assertEqual(len(a), 4)
        self.assertEqual(len(a.data), 4)
        self.assertGreaterEqual(a.capacity, 4)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])

        request = a.capacity * 2
        a.grow(request)
        self.assertEqual(len(a), 4)
        self.assertEqual(len(a.data), 4)
        self.assertGreaterEqual(a.capacity, request)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])

        request = a.capacity * 2
        a.grow(request, copy=False)
        self.assertEqual(len(a), 0)
        self.assertEqual(len(a.data), 0)
        self.assertGreaterEqual(a.capacity, request)
        self.assertEqual(a.data.tolist(), [])

    def test_resize(self):
        a = DynamicArray(0, numpy.dtype(numpy.int8))
        a.extend([0, 1, 4, 9])
        self.assertEqual(len(a), 4)
        self.assertEqual(len(a.data), 4)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])

        a.resize(2)
        self.assertEqual(len(a), 2)
        self.assertEqual(len(a.data), 2)
        self.assertEqual(a.data.tolist(), [0, 1])

        request = a.capacity * 2
        a.resize(request)
        self.assertEqual(len(a), request)
        self.assertEqual(len(a.data), request)
        self.assertGreaterEqual(a.capacity, request)
        self.assertEqual(a.data.tolist()[:2], [0, 1])

    def test_append(self):
        a = DynamicArray(0, numpy.dtype(numpy.int8))
        self.assertEqual(len(a), 0)
        self.assertEqual(len(a.data), 0)

        data = a.append(0)
        self.assertEqual(data, 0)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(a), 1)
        self.assertEqual(a.data.tolist(), [0])

        data = a.append(1)
        self.assertEqual(data, 1)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(a), 2)
        self.assertEqual(a.data.tolist(), [0, 1])

        data = a.append(4)
        self.assertEqual(data, 4)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(a), 3)
        self.assertEqual(a.data.tolist(), [0, 1, 4])

        data = a.append(9)
        self.assertEqual(data, 9)
        self.assertIsInstance(data, numpy.int8)
        self.assertEqual(len(a), 4)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])

    def test_extend(self):
        a = DynamicArray(0, numpy.dtype(numpy.int8))
        self.assertEqual(len(a), 0)
        self.assertEqual(len(a.data), 0)

        a.extend([0, 1, 4, 9])
        self.assertEqual(len(a), 4)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])

        a.extend([2, 4, 6, 8])
        self.assertEqual(len(a), 8)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9, 2, 4, 6, 8])

        a.extend(numpy.arange(4))
        self.assertEqual(len(a), 12)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9, 2, 4, 6, 8, 0, 1, 2, 3])

    def test_clear(self):
        a = DynamicArray(0, numpy.dtype(numpy.int8))
        a.extend([0, 1, 4, 9])
        self.assertEqual(len(a), 4)
        self.assertEqual(a.data.tolist(), [0, 1, 4, 9])

        a.clear()
        self.assertEqual(len(a), 0)
        self.assertEqual(a.data.tolist(), [])


if __name__ == '__main__':
    from tests.run import run
    run('__main__')
